# distutils: language = c++
# distutils: libraries = gmp gmpxx ppl m
#*****************************************************************************
#       Copyright (C) 2010-2014 Volker Braun  <vbraun.name@gmail.com>
#                     2011 Simon King <simon.king@uni-jena.de>
#                     2011-2017 Jeroen Demeyer <jdemeyer@cage.ugent.be>
#                     2012 Risan <ptrrsn.1@gmail.com>
#                     2013 Julien Puydt <julien.puydt@laposte.net>
#                     2013 Travis Scrimshaw <tscrim@ucdavis.edu>
#                     2015 André Apitzsch <andre.apitzsch@etit.tu-chemnitz.de>
#                     2016 Jori Mäntysalo <jori.mantysalo@uta.fi>
#                     2016 Matthias Koeppe <mkoeppe@math.ucdavis.edu>
#                     2016-2017 Frédéric Chapoton <chapoton@math.univ-lyon1.fr>
#                     2016-2018 Vincent Delecroix <vincent.delecroix@labri.fr>
#                     2017-2018 Vincent Klein <vinklein@gmail.com>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#  as published by the Free Software Foundation; either version 3 of
#  the License, or (at youroption) any later version.
#                  http://www.gnu.org/licenses/
#*****************************************************************************

from cysignals.signals cimport sig_on, sig_off
from gmpy2 cimport import_gmpy2, GMPy_MPQ_From_mpz
from cython.operator cimport dereference as deref

# PPL can use floating-point arithmetic to compute integers
cdef extern from "ppl.hh" namespace "Parma_Polyhedra_Library":
    cdef void set_rounding_for_PPL()
    cdef void restore_pre_PPL_rounding()

# initialize gmpy2 C API
import_gmpy2()

# but with PPL's rounding the gsl will be very unhappy; must turn off!
restore_pre_PPL_rounding()


cdef class MIP_Problem(object):
    r"""
    wrapper for PPL's MIP_Problem class

    An object of the class MIP_Problem represents a Mixed Integer
    (Linear) Program problem.

    INPUT:

    - ``dim`` -- integer
    - ``args`` -- an array of the defining data of the MIP_Problem.
      For each element, any one of the following is accepted:

      * A :class:`Constraint_System`.

      * A :class:`Linear_Expression`.

    OUTPUT:

    A :class:`MIP_Problem`.

    Examples:

        >>> from ppl import Variable, Constraint_System, MIP_Problem
        >>> x = Variable(0)
        >>> y = Variable(1)
        >>> cs = Constraint_System()
        >>> cs.insert(x >= 0)
        >>> cs.insert(y >= 0)
        >>> cs.insert(3 * x + 5 * y <= 10)
        >>> m = MIP_Problem(2, cs, x + y)
        >>> m
        MIP Problem (maximization, 2 variables, 3 constraints)
        >>> m.optimal_value()
        mpq(10,3)
        >>> float(_)
        3.333333333333333
        >>> m.optimizing_point()
        point(10/3, 0/3)
    """
    def __repr__(self):
        """
        String representation of MIP Problem.

        Examples:

        >>> from ppl import Variable, Constraint_System, MIP_Problem
        >>> x = Variable(0)
        >>> y = Variable(1)
        >>> cs = Constraint_System()
        >>> cs.insert( x >= 0 )
        >>> cs.insert( y >= 0 )
        >>> cs.insert( 3 * x + 5 * y <= 10 )
        >>> m = MIP_Problem(2, cs, x + y)
        >>> m
        MIP Problem (maximization, 2 variables, 3 constraints)
        """
        dim = self.space_dimension()
        ncs = sum(1 for _ in self)
        return 'MIP Problem ({}, {} variable{}, {} constraint{})'.format(
                self.optimization_mode(),
                dim,
                's' if dim > 1 else '',
                ncs,
                's' if ncs > 1 else '')

    def __cinit__(self, PPL_dimension_type dim = 0, *args):
        """
        The Cython constructor.

        Tests:

        >>> from ppl import Variable, Constraint_System, MIP_Problem
        >>> MIP_Problem(0)
        A MIP_Problem
        Maximize: 0
        Subject to constraints
        >>> x = Variable(0)
        >>> y = Variable(1)
        >>> cs = Constraint_System()
        >>> cs.insert(x + y <= 2)
        >>> M = MIP_Problem(2, cs, 0)
        >>> M = MIP_Problem(2, cs, x)
        >>> M = MIP_Problem(2, None, None)
        Traceback (most recent call last):
        ...
        TypeError: Cannot convert NoneType to ppl.Constraint_System
        >>> M = MIP_Problem(2, cs, 'hey')
        Traceback (most recent call last):
        ...
        TypeError: unable to convert 'hey' to an integer
        >>> M = MIP_Problem(2, cs, x, 'middle')
        Traceback (most recent call last):
        ...
        ValueError: unknown mode 'middle'
        """
        if not args:
            self.thisptr = new PPL_MIP_Problem(dim)
            return
        elif len(args) == 1:
            raise ValueError('cannot initialize from {}'.format(args))

        cdef Constraint_System cs = <Constraint_System?>args[0]
        cdef Linear_Expression obj
        try:
            obj = <Linear_Expression?> args[1]
        except TypeError:
            obj = Linear_Expression(args[1])

        cdef PPL_Optimization_Mode mode = MAXIMIZATION
        if len(args) == 3:
            if args[2] == 'maximization':
                mode = MAXIMIZATION
            elif args[2] == 'minimization':
                mode = MINIMIZATION
            else:
                raise ValueError('unknown mode {!r}'.format(args[2]))
        self.thisptr = new PPL_MIP_Problem(dim, cs.thisptr[0], obj.thisptr[0], mode)

    def __dealloc__(self):
        """
        The Cython destructor
        """
        del self.thisptr

    def __iter__(self):
        r"""
        Iterator through the constraints
        Tests:

        >>> from ppl import Variable, MIP_Problem
        >>> x = Variable(0)
        >>> y = Variable(1)
        >>> M = MIP_Problem(2)
        >>> for c in M: print(c)
        >>> M.add_constraint(x + y <= 5)
        >>> for c in M: print(c)
        -x0-x1+5>=0
        >>> M.add_constraint(3*x - 18*y >= -2)
        >>> for c in M: print(c)
        -x0-x1+5>=0
        3*x0-18*x1+2>=0
        >>> M = MIP_Problem(1)
        >>> M.add_constraint(x <= 5)
        >>> it = M.__iter__()
        >>> next(it)
        -x0+5>=0
        >>> next(it)
        Traceback (most recent call last):
        ...
        StopIteration
        """
        cdef PPL_mip_iterator *mip_csi_ptr = new PPL_mip_iterator(self.thisptr[0].constraints_begin())
        try:
            while mip_csi_ptr[0] != self.thisptr[0].constraints_end():
                yield _wrap_Constraint(deref(mip_csi_ptr[0].inc(1)))
        finally:
            del mip_csi_ptr

    def constraints(self):
        r"""
        Return the constraints of this MIP

        The output is an instance of :class:`Constraint_System`.

        Examples:

        >>> from ppl import Variable, MIP_Problem
        >>> x = Variable(0)
        >>> y = Variable(1)
        >>> M = MIP_Problem(2)
        >>> M.add_constraint(x + y <= 5)
        >>> M.add_constraint(3*x - 18*y >= -2)
        >>> M.constraints()
        Constraint_System {-x0-x1+5>=0, 3*x0-18*x1+2>=0}

        Note that modifying the output of this method will not modify the
        underlying MIP problem object:

        >>> cs = M.constraints()
        >>> cs.insert(x <= 3)
        >>> cs
        Constraint_System {-x0-x1+5>=0, 3*x0-18*x1+2>=0, -x0+3>=0}
        >>> M.constraints()
        Constraint_System {-x0-x1+5>=0, 3*x0-18*x1+2>=0}
        """
        cdef Constraint_System c = Constraint_System(None)
        cdef PPL_Constraint_System* cs = new PPL_Constraint_System()
        cdef PPL_mip_iterator* mip_it = new PPL_mip_iterator(self.thisptr[0].constraints_begin())

        while mip_it[0] != self.thisptr[0].constraints_end():
            cs[0].insert(deref(mip_it[0]))
            mip_it[0].inc(1)
        c.thisptr = cs
        del mip_it
        return c

    def optimization_mode(self):
        """
        Return the optimization mode used in the MIP_Problem.

        It will return "maximization" if the MIP_Problem was set
        to MAXIMIZATION mode, and "minimization" otherwise.

        Examples:

            >>> from ppl import MIP_Problem
            >>> m = MIP_Problem()
            >>> m.optimization_mode()
            'maximization'
        """
        if self.thisptr.optimization_mode() == MAXIMIZATION:
            return "maximization"
        elif self.thisptr.optimization_mode() == MINIMIZATION:
            return "minimization"

    def optimal_value(self):
        """
        Return the optimal value of the MIP_Problem. ValueError thrown if self does not
        have an optimizing point, i.e., if the MIP problem is unbounded or not satisfiable.

        Examples:

        >>> from ppl import Variable, Constraint_System, MIP_Problem
        >>> x = Variable(0)
        >>> y = Variable(1)
        >>> cs = Constraint_System()
        >>> cs.insert( x >= 0 )
        >>> cs.insert( y >= 0 )
        >>> cs.insert( 3 * x + 5 * y <= 10 )
        >>> m = MIP_Problem(2, cs, x + y)
        >>> m.optimal_value()
        mpq(10,3)
        >>> cs = Constraint_System()
        >>> cs.insert( x >= 0 )
        >>> m = MIP_Problem(1, cs, x + x )
        >>> m.optimal_value()
        Traceback (most recent call last):
        ...
        ValueError: PPL::MIP_Problem::optimizing_point():
        *this ... have an optimizing point.
        """
        cdef PPL_Coefficient sup_n
        cdef PPL_Coefficient sup_d

        sig_on()
        try:
            self.thisptr.optimal_value(sup_n, sup_d)
        finally:
            sig_off()
        return GMPy_MPQ_From_mpz(sup_n.get_mpz_t(), sup_d.get_mpz_t())

    def space_dimension(self):
        """
        Return the space dimension of the MIP_Problem.

        Examples:

        >>> from ppl import Variable, Constraint_System, MIP_Problem
        >>> x = Variable(0)
        >>> y = Variable(1)
        >>> cs = Constraint_System()
        >>> cs.insert( x >= 0)
        >>> cs.insert( y >= 0 )
        >>> cs.insert( 3 * x + 5 * y <= 10 )
        >>> m = MIP_Problem(2, cs, x + y)
        >>> m.space_dimension()
        2
        """
        return self.thisptr.space_dimension()

    def objective_function(self):
        """
        Return the optimal value of the MIP_Problem.

        Examples:

        >>> from ppl import Variable, Constraint_System, MIP_Problem
        >>> x = Variable(0)
        >>> y = Variable(1)
        >>> cs = Constraint_System()
        >>> cs.insert( x >= 0)
        >>> cs.insert( y >= 0 )
        >>> cs.insert( 3 * x + 5 * y <= 10 )
        >>> m = MIP_Problem(2, cs, x + y)
        >>> m.objective_function()
        x0+x1
        """
        rc = Linear_Expression()
        rc.thisptr[0] = self.thisptr.objective_function()
        return rc

    def clear(self):
        """
        Reset the MIP_Problem to be equal to the trivial MIP_Problem.

        Examples:

        >>> from ppl import Variable, Constraint_System, MIP_Problem
        >>> x = Variable(0)
        >>> y = Variable(1)
        >>> cs = Constraint_System()
        >>> cs.insert( x >= 0)
        >>> cs.insert( y >= 0 )
        >>> cs.insert( 3 * x + 5 * y <= 10 )
        >>> m = MIP_Problem(2, cs, x + y)
        >>> m.objective_function()
        x0+x1
        >>> m.clear()
        >>> m.objective_function()
        0
        """
        self.thisptr.clear()

    def add_space_dimensions_and_embed(self, PPL_dimension_type m):
        """
        Adds m new space dimensions and embeds the old MIP problem in the new vector space.

        Examples:

        >>> from ppl import Variable, Constraint_System, MIP_Problem
        >>> x = Variable(0)
        >>> y = Variable(1)
        >>> cs = Constraint_System()
        >>> cs.insert( x >= 0)
        >>> cs.insert( y >= 0 )
        >>> cs.insert( 3 * x + 5 * y <= 10 )
        >>> m = MIP_Problem(2, cs, x + y)
        >>> m.add_space_dimensions_and_embed(5)
        >>> m.space_dimension()
        7
        """
        sig_on()
        self.thisptr.add_space_dimensions_and_embed(m)
        sig_off()

    def add_constraint(self, Constraint c):
        """
        Adds a copy of constraint c to the MIP problem.

        Examples:

        >>> from ppl import Variable, MIP_Problem
        >>> x = Variable(0)
        >>> y = Variable(1)
        >>> m = MIP_Problem()
        >>> m.add_space_dimensions_and_embed(2)
        >>> m.add_constraint(x >= 0)
        >>> m.add_constraint(y >= 0)
        >>> m.add_constraint(3 * x + 5 * y <= 10)
        >>> m.set_objective_function(x + y)
        >>> m.optimal_value()
        mpq(10,3)

        Tests:

        >>> z = Variable(2)
        >>> m.add_constraint(z >= -3)
        Traceback (most recent call last):
        ...
        ValueError: PPL::MIP_Problem::add_constraint(c):
        c.space_dimension() == 3 exceeds this->space_dimension == 2.
        """
        sig_on()
        try:
            self.thisptr.add_constraint(c.thisptr[0])
        finally:
            sig_off()

    def add_constraints(self, Constraint_System cs):
        """
        Adds a copy of the constraints in cs to the MIP problem.

        Examples:

        >>> from ppl import Variable, Constraint_System, MIP_Problem
        >>> x = Variable(0)
        >>> y = Variable(1)
        >>> cs = Constraint_System()
        >>> cs.insert(x >= 0)
        >>> cs.insert(y >= 0)
        >>> cs.insert(3 * x + 5 * y <= 10)
        >>> m = MIP_Problem(2)
        >>> m.set_objective_function(x + y)
        >>> m.add_constraints(cs)
        >>> m.optimal_value()
        mpq(10,3)

        Tests:

        >>> p = Variable(9)
        >>> cs.insert(p >= -3)
        >>> m.add_constraints(cs)
        Traceback (most recent call last):
        ...
        ValueError: PPL::MIP_Problem::add_constraints(cs):
        cs.space_dimension() == 10 exceeds this->space_dimension() == 2.
        """
        sig_on()
        try:
            self.thisptr.add_constraints(cs.thisptr[0])
        finally:
            sig_off()

    def add_to_integer_space_dimensions(self, Variables_Set i_vars):
        """
        Sets the variables whose indexes are in set `i_vars` to be integer space dimensions.

        Examples:

        >>> from ppl import Variable, Variables_Set, Constraint_System, MIP_Problem
        >>> x = Variable(0)
        >>> y = Variable(1)
        >>> cs = Constraint_System()
        >>> cs.insert( x >= 0)
        >>> cs.insert( y >= 0 )
        >>> cs.insert( 3 * x + 5 * y <= 10 )
        >>> m = MIP_Problem(2)
        >>> m.set_objective_function(x + y)
        >>> m.add_constraints(cs)
        >>> i_vars = Variables_Set(x, y)
        >>> m.add_to_integer_space_dimensions(i_vars)
        >>> m.optimal_value()
        mpq(3,1)
        """
        sig_on()
        try:
            self.thisptr.add_to_integer_space_dimensions(i_vars.thisptr[0])
        finally:
            sig_off()

    def set_objective_function(self, obj):
        """
        Sets the objective function to obj.

        Examples:

        >>> from ppl import Variable, MIP_Problem
        >>> x = Variable(0)
        >>> y = Variable(1)
        >>> m = MIP_Problem()
        >>> m.add_space_dimensions_and_embed(2)
        >>> m.add_constraint(x >= 0)
        >>> m.add_constraint(y >= 0)
        >>> m.add_constraint(3 * x + 5 * y <= 10)
        >>> m.set_objective_function(x + y)
        >>> m.optimal_value()
        mpq(10,3)

        Tests:

        >>> z = Variable(2)
        >>> m.set_objective_function(x + y + z)
        Traceback (most recent call last):
        ...
        ValueError: PPL::MIP_Problem::set_objective_function(obj):
        obj.space_dimension() == 3 exceeds this->space_dimension == 2.

        >>> M = MIP_Problem(1)
        >>> M.set_objective_function(Variable(0))
        """
        if isinstance(obj, Variable):
            obj = Linear_Expression(obj)
        elif not isinstance(obj, Linear_Expression):
            raise ValueError('not an objective function')
        self.thisptr.set_objective_function((<Linear_Expression> obj).thisptr[0])

    def set_optimization_mode(self, mode):
        """
        Sets the optimization mode to mode.

        Examples:

        >>> from ppl import MIP_Problem
        >>> m = MIP_Problem()
        >>> m.optimization_mode()
        'maximization'
        >>> m.set_optimization_mode('minimization')
        >>> m.optimization_mode()
        'minimization'

        Tests:

        >>> m.set_optimization_mode('max')
        Traceback (most recent call last):
        ...
        ValueError: Unknown value: mode=max.
        """
        if mode == 'minimization':
            self.thisptr.set_optimization_mode(MINIMIZATION)
        elif mode == 'maximization':
            self.thisptr.set_optimization_mode(MAXIMIZATION)
        else:
            raise ValueError('Unknown value: mode='+str(mode)+'.')

    def is_satisfiable(self):
        """
        Check if the MIP_Problem is satisfiable

        Examples:

        >>> from ppl import Variable, MIP_Problem
        >>> x = Variable(0)
        >>> y = Variable(1)
        >>> m = MIP_Problem()
        >>> m.add_space_dimensions_and_embed(2)
        >>> m.add_constraint(x >= 0)
        >>> m.add_constraint(y >= 0)
        >>> m.add_constraint(3 * x + 5 * y <= 10)
        >>> m.is_satisfiable()
        True
        """
        return self.thisptr.is_satisfiable()

    def evaluate_objective_function(self, Generator evaluating_point):
        """
        Return the result of evaluating the objective function on evaluating_point. ValueError thrown
        if self and evaluating_point are dimension-incompatible or if the generator evaluating_point is not a point.

        Examples:

        >>> from ppl import Variable, MIP_Problem, Generator
        >>> x = Variable(0)
        >>> y = Variable(1)
        >>> m = MIP_Problem()
        >>> m.add_space_dimensions_and_embed(2)
        >>> m.add_constraint(x >= 0)
        >>> m.add_constraint(y >= 0)
        >>> m.add_constraint(3 * x + 5 * y <= 10)
        >>> m.set_objective_function(x + y)
        >>> g = Generator.point(5 * x - 2 * y, 7)
        >>> m.evaluate_objective_function(g)
        mpq(3,7)
        >>> z = Variable(2)
        >>> g = Generator.point(5 * x - 2 * z, 7)
        >>> m.evaluate_objective_function(g)
        Traceback (most recent call last):
        ...
        ValueError: PPL::MIP_Problem::evaluate_objective_function(p, n, d):
        *this and p are dimension incompatible.
        """
        cdef PPL_Coefficient sup_n
        cdef PPL_Coefficient sup_d

        sig_on()
        try:
            self.thisptr.evaluate_objective_function(evaluating_point.thisptr[0], sup_n, sup_d)
        finally:
            sig_off()

        return GMPy_MPQ_From_mpz(sup_n.get_mpz_t(), sup_d.get_mpz_t())

    def solve(self):
        """
        Optimizes the MIP_Problem

        Examples:

        >>> from ppl import Variable, MIP_Problem
        >>> x = Variable(0)
        >>> y = Variable(1)
        >>> m = MIP_Problem()
        >>> m.add_space_dimensions_and_embed(2)
        >>> m.add_constraint(x >= 0)
        >>> m.add_constraint(y >= 0)
        >>> m.add_constraint(3 * x + 5 * y <= 10)
        >>> m.set_objective_function(x + y)
        >>> m.solve()
        {'status': 'optimized'}
        """
        sig_on()
        try:
            tmp = self.thisptr.solve()
        finally:
            sig_off()
        if tmp == UNFEASIBLE_MIP_PROBLEM:
            return {'status': 'unfeasible'}
        elif tmp == UNBOUNDED_MIP_PROBLEM:
            return {'status': 'unbounded'}
        else:
            return {'status': 'optimized'}

    def optimizing_point(self):
        """
        Returns an optimal point for the MIP_Problem, if it exists.

        Examples:

        >>> from ppl import Variable, MIP_Problem
        >>> x = Variable(0)
        >>> y = Variable(1)
        >>> m = MIP_Problem()
        >>> m.add_space_dimensions_and_embed(2)
        >>> m.add_constraint(x >= 0)
        >>> m.add_constraint(y >= 0)
        >>> m.add_constraint(3 * x + 5 * y <= 10)
        >>> m.set_objective_function(x + y)
        >>> m.optimizing_point()
        point(10/3, 0/3)
        """
        cdef PPL_Generator *g
        sig_on()
        try:
            g = new PPL_Generator(self.thisptr[0].optimizing_point())
        finally:
            sig_off()
        return _wrap_Generator(g[0])



# distutils: language = c++
# distutils: libraries = gmp gmpxx ppl m
#*****************************************************************************
#       Copyright (C) 2010-2014 Volker Braun  <vbraun.name@gmail.com>
#                     2011 Simon King <simon.king@uni-jena.de>
#                     2011-2017 Jeroen Demeyer <jdemeyer@cage.ugent.be>
#                     2012 Risan <ptrrsn.1@gmail.com>
#                     2013 Julien Puydt <julien.puydt@laposte.net>
#                     2013 Travis Scrimshaw <tscrim@ucdavis.edu>
#                     2015 André Apitzsch <andre.apitzsch@etit.tu-chemnitz.de>
#                     2016 Jori Mäntysalo <jori.mantysalo@uta.fi>
#                     2016 Matthias Koeppe <mkoeppe@math.ucdavis.edu>
#                     2016-2017 Frédéric Chapoton <chapoton@math.univ-lyon1.fr>
#                     2016-2020 Vincent Delecroix <vincent.delecroix@labri.fr>
#                     2017-2018 Vincent Klein <vinklein@gmail.com>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#  as published by the Free Software Foundation; either version 3 of
#  the License, or (at youroption) any later version.
#                  http://www.gnu.org/licenses/
#*****************************************************************************

from gmpy2 cimport GMPy_MPZ_From_mpz, import_gmpy2
from cython.operator cimport dereference as deref
from cpython.object cimport Py_LT, Py_LE, Py_EQ, Py_NE, Py_GT, Py_GE

from .congruence cimport Congruence

# PPL can use floating-point arithmetic to compute integers
cdef extern from "ppl.hh" namespace "Parma_Polyhedra_Library":
    cdef void set_rounding_for_PPL()
    cdef void restore_pre_PPL_rounding()

# initialize gmpy2 C API
import_gmpy2()

# but with PPL's rounding the gsl will be very unhappy; must turn off!
restore_pre_PPL_rounding()


def _dummy():
    raise ValueError


cdef class Constraint(object):
    r"""
    Wrapper for PPL's ``Constraint`` class.

    An object of the class ``Constraint`` is either:

    * an equality :math:`\sum_{i=0}^{n-1} a_i x_i + b = 0`

    * a non-strict inequality :math:`\sum_{i=0}^{n-1} a_i x_i + b \geq 0`

    * a strict inequality :math:`\sum_{i=0}^{n-1} a_i x_i + b > 0`

    where :math:`n` is the dimension of the space, :math:`a_i` is the integer
    coefficient of variable :math:`x_i`, and :math:`b_i` is the integer
    inhomogeneous term.

    INPUT/OUTPUT:

    You construct constraints by writing inequalities in
    :class:`Linear_Expression`. Do not attempt to manually construct
    constraints.

    Examples:

    >>> from ppl import Variable, Linear_Expression
    >>> x = Variable(0)
    >>> y = Variable(1)
    >>> 5*x-2*y >  x+y-1
    4*x0-3*x1+1>0
    >>> 5*x-2*y >= x+y-1
    4*x0-3*x1+1>=0
    >>> 5*x-2*y == x+y-1
    4*x0-3*x1+1==0
    >>> 5*x-2*y <= x+y-1
    -4*x0+3*x1-1>=0
    >>> 5*x-2*y <  x+y-1
    -4*x0+3*x1-1>0
    >>> x > 0
    x0>0

    Special care is needed if the left hand side is a constant:

    >>> 0 == 1    # watch out!
    False
    >>> Linear_Expression(0) == 1
    mpz(-1)==0
    """
    def __init__(self, arg=None):
        if arg is None:
            self.thisptr = new PPL_Constraint()
        elif isinstance(arg, Constraint):
            self.thisptr = new PPL_Constraint((<Constraint> arg).thisptr[0])
        else:
            raise TypeError("invalid argument for Constraint")

    def __cinit__(self):
        """
        The Cython constructor.

        See :class:`Constraint` for documentation.

        Tests:

            >>> from ppl import Variable
            >>> x = Variable(0)
            >>> x>0   # indirect doctest
            x0>0
        """
        self.thisptr = NULL

    def __dealloc__(self):
        """
        The Cython destructor.
        """
        del self.thisptr

    def __hash__(self):
        r"""
        Tests:

        >>> import ppl
        >>> hash(ppl.Variable(0) == 3)
        Traceback (most recent call last):
        ...
        TypeError: Constraint unhashable
        """
        raise TypeError('Constraint unhashable')

    def __repr__(self):
        """
        Return a string representation of the constraint.

        OUTPUT:

        String.

        Examples:

        >>> from ppl import Variable
        >>> x = Variable(0)
        >>> y = Variable(1)
        >>> (2*x-y+5 >  x).__repr__()
        'x0-x1+5>0'
        >>> (2*x-y+5 == x).__repr__()
        'x0-x1+5==0'
        >>> (2*x-y+5 >= x).__repr__()
        'x0-x1+5>=0'
        """
        e = sum(self.coefficient(x)*x
                for x in (Variable(i)
                          for i in range(self.space_dimension())))
        e += self.inhomogeneous_term()
        s = repr(e)
        t = self.thisptr.type()
        if t == EQUALITY:
            s += '==0'
        elif t == NONSTRICT_INEQUALITY:
            s += '>=0'
        elif t == STRICT_INEQUALITY:
            s += '>0'
        else:
            raise RuntimeError
        return s

    def space_dimension(self):
        r"""
        Return the dimension of the vector space enclosing ``self``.

        OUTPUT:

        Integer.

        Examples:

        >>> from ppl import Variable
        >>> x = Variable(0)
        >>> y = Variable(1)
        >>> (x>=0).space_dimension()
        1
        >>> (y==1).space_dimension()
        2
        """
        return self.thisptr.space_dimension()

    def type(self):
        r"""
        Return the constraint type of ``self``.

        OUTPUT:

        String. One of ``'equality'``, ``'nonstrict_inequality'``, or
        ``'strict_inequality'``.

        Examples:

            >>> from ppl import Variable
            >>> x = Variable(0)
            >>> (x==0).type()
            'equality'
            >>> (x>=0).type()
            'nonstrict_inequality'
            >>> (x>0).type()
            'strict_inequality'
        """
        t = self.thisptr.type()
        if t == EQUALITY:
            return 'equality'
        elif t == NONSTRICT_INEQUALITY:
            return 'nonstrict_inequality'
        elif t == STRICT_INEQUALITY:
            return 'strict_inequality'
        else:
            raise RuntimeError

    def is_equality(self):
        r"""
        Test whether ``self`` is an equality.

        OUTPUT:

        Boolean. Returns ``True`` if and only if ``self`` is an
        equality constraint.

        Examples:

        >>> from ppl import Variable
        >>> x = Variable(0)
        >>> (x==0).is_equality()
        True
        >>> (x>=0).is_equality()
        False
        >>> (x>0).is_equality()
        False
        """
        return self.thisptr.is_equality()

    def is_inequality(self):
        r"""
        Test whether ``self`` is an inequality.

        OUTPUT:

        Boolean. Returns ``True`` if and only if ``self`` is an
        inequality constraint, either strict or non-strict.

        Examples:

        >>> from ppl import Variable
        >>> x = Variable(0)
        >>> (x==0).is_inequality()
        False
        >>> (x>=0).is_inequality()
        True
        >>> (x>0).is_inequality()
        True
        """
        return self.thisptr.is_inequality()

    def is_nonstrict_inequality(self):
        r"""
        Test whether ``self`` is a non-strict inequality.

        OUTPUT:

        Boolean. Returns ``True`` if and only if ``self`` is an
        non-strict inequality constraint.

        Examples:

        >>> from ppl import Variable
        >>> x = Variable(0)
        >>> (x==0).is_nonstrict_inequality()
        False
        >>> (x>=0).is_nonstrict_inequality()
        True
        >>> (x>0).is_nonstrict_inequality()
        False
        """
        return self.thisptr.is_nonstrict_inequality()

    def is_strict_inequality(self):
        r"""
        Test whether ``self`` is a strict inequality.

        OUTPUT:

        Boolean. Returns ``True`` if and only if ``self`` is an
        strict inequality constraint.

        Examples:

        >>> from ppl import Variable
        >>> x = Variable(0)
        >>> (x==0).is_strict_inequality()
        False
        >>> (x>=0).is_strict_inequality()
        False
        >>> (x>0).is_strict_inequality()
        True
        """
        return self.thisptr.is_strict_inequality()

    def coefficient(self, Variable v):
        """
        Return the coefficient of the variable ``v``.

        INPUT:

        - ``v`` -- a :class:`Variable`.

        OUTPUT:

        An integer.

        Examples:

        >>> from ppl import Variable
        >>> x = Variable(0)
        >>> ineq = 3*x+1 > 0
        >>> ineq.coefficient(x)
        mpz(3)
        >>> y = Variable(1)
        >>> ineq = 3**50 * y + 2 > 1
        >>> str(ineq.coefficient(y))
        '717897987691852588770249'
        >>> ineq.coefficient(x)
        mpz(0)
        """
        return GMPy_MPZ_From_mpz(self.thisptr.coefficient(v.thisptr[0]).get_mpz_t())

    def coefficients(self):
        """
        Return the coefficients of the constraint.

        See also :meth:`coefficient`.

        OUTPUT:

        A tuple of integers of length :meth:`space_dimension`.

        Examples:

        >>> from ppl import Variable
        >>> x = Variable(0);  y = Variable(1)
        >>> ineq = ( 3*x+5*y+1 ==  2);  ineq
        3*x0+5*x1-1==0
        >>> ineq.coefficients()
        (mpz(3), mpz(5))
        """
        cdef int d = self.space_dimension()
        cdef int i
        coeffs = []
        for i in range(0, d):
            coeffs.append(GMPy_MPZ_From_mpz(self.thisptr.coefficient(PPL_Variable(i)).get_mpz_t()))
        return tuple(coeffs)

    def inhomogeneous_term(self):
        """
        Return the inhomogeneous term of the constraint.

        OUTPUT:

        Integer.

        Examples:

        >>> from ppl import Variable
        >>> y = Variable(1)
        >>> ineq = 10+y > 9
        >>> ineq
        x1+1>0
        >>> ineq.inhomogeneous_term()
        mpz(1)
        >>> ineq = 2**66 + y > 0
        >>> str(ineq.inhomogeneous_term())
        '73786976294838206464'
        """
        return GMPy_MPZ_From_mpz(self.thisptr.inhomogeneous_term().get_mpz_t())

    def is_tautological(self):
        r"""
        Test whether ``self`` is a tautological constraint.

        A tautology can have either one of the following forms:

        * an equality: :math:`\sum 0 x_i + 0 = 0`,

        * a non-strict inequality: :math:`\sum 0 x_i + b \geq 0` with `b\geq 0`, or

        * a strict inequality: :math:`\sum 0 x_i + b > 0` with `b> 0`.

        OUTPUT:

        Boolean. Returns ``True`` if and only if ``self`` is a
        tautological constraint.

        Examples:

        >>> from ppl import Variable
        >>> x = Variable(0)
        >>> (x==0).is_tautological()
        False
        >>> (0*x>=0).is_tautological()
        True
        """
        return self.thisptr.is_tautological()

    def is_inconsistent(self):
        r"""
        Test whether ``self`` is an inconsistent constraint, that is, always false.

        An inconsistent constraint can have either one of the
        following forms:

        * an equality: :math:`\sum 0 x_i + b = 0` with `b\not=0`,

        * a non-strict inequality: :math:`\sum 0 x_i + b \geq 0` with `b< 0`, or

        * a strict inequality: :math:`\sum 0 x_i + b > 0` with `b\leq 0`.

        OUTPUT:

        Boolean. Returns ``True`` if and only if ``self`` is an
        inconsistent constraint.

        Examples:

        >>> from ppl import Variable
        >>> x = Variable(0)
        >>> (x==1).is_inconsistent()
        False
        >>> (0*x>=1).is_inconsistent()
        True
        """
        return self.thisptr.is_inconsistent()

    def is_equivalent_to(self, Constraint c):
        r"""
        Test whether ``self`` and ``c`` are equivalent.

        INPUT:

        - ``c`` -- a :class:`Constraint`.

        OUTPUT:

        Boolean. Returns ``True`` if and only if ``self`` and ``c``
        are equivalent constraints.

        Note that constraints having different space dimensions are
        not equivalent. However, constraints having different types
        may nonetheless be equivalent, if they both are tautologies or
        inconsistent.

        Examples:

        >>> from ppl import Variable, Linear_Expression
        >>> x = Variable(0)
        >>> y = Variable(1)
        >>> (x > 0).is_equivalent_to(Linear_Expression(0) < x)
        True
        >>> (x > 0).is_equivalent_to(0*y < x)
        False
        >>> (0*x > 1).is_equivalent_to(0*x == -2)
        True
        """
        return self.thisptr.is_equivalent_to(c.thisptr[0])

    def ascii_dump(self):
        r"""
        Write an ASCII dump to stderr.

        Examples:

        >>> cmd  = 'from ppl import Linear_Expression, Variable\n'
        >>> cmd += 'x = Variable(0)\n'
        >>> cmd += 'y = Variable(1)\n'
        >>> cmd += 'e = (3*x+2*y+1 > 0)\n'
        >>> cmd += 'e.ascii_dump()\n'
        >>> import subprocess, sys
        >>> proc = subprocess.Popen([sys.executable, '-c', cmd], stderr=subprocess.PIPE)
        >>> out, err = proc.communicate()
        >>> print(str(err.decode('ascii')))
        size 4 1 3 2 -1 ...
        <BLANKLINE>
        """
        self.thisptr.ascii_dump()

    def __reduce__(self):
        """
        Pickle object.

        Examples:

        >>> from ppl import Variable
        >>> from pickle import loads, dumps
        >>> x = Variable(0)
        >>> y = Variable(1)
        >>> loads(dumps(3*x+2*y+1>=5))
        3*x0+2*x1-4>=0
        >>> loads(dumps(3*x+2*y+1>5))
        3*x0+2*x1-4>0
        >>> loads(dumps(3*x+2*y+1==5))
        3*x0+2*x1-4==0
        """
        le = Linear_Expression(self.coefficients(), self.inhomogeneous_term())
        if self.is_nonstrict_inequality():
            return (inequality, (le, ))
        elif self.is_strict_inequality():
            return (strict_inequality, (le, ))
        elif self.is_equality():
            return (equation, (le, ))
        else:
            raise RuntimeError

    def permute_space_dimensions(self, cycle):
        """
        Permute the coordinates according to ``cycle``.

        >>> from ppl import Variable
        >>> x = Variable(0); y = Variable(1); z = Variable(2)
        >>> l = 2*x - y + 3*z
        >>> ieq = l >= 5
        >>> ieq.permute_space_dimensions([2, 1, 0])
        >>> ieq
        -x0+3*x1+2*x2-5>=0
        """
        cdef cppvector[PPL_Variable] cpp_cycle
        cdef int i
        for i in cycle:
            cpp_cycle.push_back(PPL_Variable(i))
        self.thisptr.permute_space_dimensions(cpp_cycle)

    def __mod__(self, m):
        r"""
        Return this equality modulo m

        >>> import ppl
        >>> x = ppl.Variable(0)
        >>> y = ppl.Variable(1)
        >>> (2*x + 3*y == 3) % 5
        2*x0+3*x1-3==0 (mod 5)

        >>> (x <= 3) % 5
        Traceback (most recent call last):
        ...
        ValueError: PPL::Congruence::Congruence(c, r):
        constraint c must be an equality.
        """
        return Congruence(self, m)


def inequality(expression):
    """
    Construct an inequality.

    INPUT:

    - ``expression`` -- a :class:`Linear_Expression`.

    OUTPUT:

    The inequality ``expression`` >= 0.

    Examples:

    >>> from ppl import Variable, inequality
    >>> y = Variable(1)
    >>> 2*y+1 >= 0
    2*x1+1>=0
    >>> inequality(2*y+1)
    2*x1+1>=0
    """
    return expression >= 0


def strict_inequality(expression):
    """
    Construct a strict inequality.

    INPUT:

    - ``expression`` -- a :class:`Linear_Expression`.

    OUTPUT:

    The inequality ``expression`` > 0.

    Examples:

    >>> from ppl import Variable, strict_inequality
    >>> y = Variable(1)
    >>> 2*y+1 > 0
    2*x1+1>0
    >>> strict_inequality(2*y+1)
    2*x1+1>0
    """
    return expression > 0


def equation(expression):
    """
    Construct an equation.

    INPUT:

    - ``expression`` -- a :class:`Linear_Expression`.

    OUTPUT:

    The equation ``expression`` == 0.

    Examples:

    >>> from ppl import Variable, equation
    >>> y = Variable(1)
    >>> 2*y+1 == 0
    2*x1+1==0
    >>> equation(2*y+1)
    2*x1+1==0
    """
    return expression == 0


cdef class Constraint_System(object):
    """
    Wrapper for PPL's ``Constraint_System`` class.

    An object of the class Constraint_System is a system of
    constraints, i.e., a multiset of objects of the class
    Constraint. When inserting constraints in a system, space
    dimensions are automatically adjusted so that all the constraints
    in the system are defined on the same vector space.

    Examples:

        >>> from ppl import Constraint_System, Variable
        >>> x = Variable(0)
        >>> y = Variable(1)
        >>> cs = Constraint_System( 5*x-2*y > 0 )
        >>> cs.insert( 6*x < 3*y )
        >>> cs.insert( x >= 2*x-7*y )
        >>> cs
        Constraint_System {5*x0-2*x1>0, ...}
        >>> cs[0]
        5*x0-2*x1>0
    """
    def __init__(self, arg=None):
        if arg is None:
            self.thisptr = new PPL_Constraint_System()
        elif isinstance(arg, Constraint):
            g = <Constraint>arg
            self.thisptr = new PPL_Constraint_System(g.thisptr[0])
        elif isinstance(arg, Constraint_System):
            gs = <Constraint_System>arg
            self.thisptr = new PPL_Constraint_System(gs.thisptr[0])
        elif isinstance(arg, (list, tuple)):
            self.thisptr = new PPL_Constraint_System()
            for constraint in arg:
                self.insert(constraint)
        else:
            raise TypeError('cannot initialize from {!r}'.format(arg))

    def __cinit__(self, arg=None):
        """
        The Cython constructor.

        See :class:`Constraint_System` for documentation.

        Tests:

        >>> from ppl import Constraint_System
        >>> Constraint_System()
        Constraint_System {}
        """
        self.thisptr = NULL

    def __dealloc__(self):
        """
        The Cython destructor.
        """
        del self.thisptr

    def __hash__(self):
        r"""
        Tests:

        >>> import ppl
        >>> hash(ppl.Constraint_System())
        Traceback (most recent call last):
        ...
        TypeError: Constraint_System unhashable
        """
        raise TypeError('Constraint_System unhashable')

    def space_dimension(self):
        r"""
        Return the dimension of the vector space enclosing ``self``.

        OUTPUT:

        Integer.

        Examples:

        >>> from ppl import Variable, Constraint_System
        >>> x = Variable(0)
        >>> cs = Constraint_System( x>0 )
        >>> cs.space_dimension()
        1
        """
        return self.thisptr.space_dimension()

    def has_equalities(self):
        r"""
        Tests whether ``self`` contains one or more equality constraints.

        OUTPUT:

        Boolean.

        Examples:

        >>> from ppl import Variable, Constraint_System
        >>> x = Variable(0)
        >>> cs = Constraint_System()
        >>> cs.insert( x>0 )
        >>> cs.insert( x<0 )
        >>> cs.has_equalities()
        False
        >>> cs.insert( x==0 )
        >>> cs.has_equalities()
        True
        """
        return self.thisptr.has_equalities()

    def has_strict_inequalities(self):
        r"""
        Tests whether ``self`` contains one or more strict inequality constraints.

        OUTPUT:

        Boolean.

        Examples:

        >>> from ppl import Variable, Constraint_System
        >>> x = Variable(0)
        >>> cs = Constraint_System()
        >>> cs.insert( x>=0 )
        >>> cs.insert( x==-1 )
        >>> cs.has_strict_inequalities()
        False
        >>> cs.insert( x>0 )
        >>> cs.has_strict_inequalities()
        True
        """
        return self.thisptr.has_strict_inequalities()

    def clear(self):
        r"""
        Removes all constraints from the constraint system and sets its
        space dimension to 0.

        Examples:

        >>> from ppl import Variable, Constraint_System
        >>> x = Variable(0)
        >>> cs = Constraint_System(x>0)
        >>> cs
        Constraint_System {x0>0}
        >>> cs.clear()
        >>> cs
        Constraint_System {}
        """
        self.thisptr.clear()

    def insert(self, Constraint c):
        """
        Insert ``c`` into the constraint system.

        INPUT:

        - ``c`` -- a :class:`Constraint`.

        Examples:

        >>> from ppl import Variable, Constraint_System
        >>> x = Variable(0)
        >>> cs = Constraint_System()
        >>> cs.insert( x>0 )
        >>> cs
        Constraint_System {x0>0}
        """
        self.thisptr.insert(c.thisptr[0])

    def empty(self):
        """
        Return ``True`` if and only if ``self`` has no constraints.

        OUTPUT:

        Boolean.

        Examples:

        >>> from ppl import Variable, Constraint_System
        >>> x = Variable(0)
        >>> cs = Constraint_System()
        >>> cs.empty()
        True
        >>> cs.insert( x>0 )
        >>> cs.empty()
        False
        """
        return self.thisptr.empty()

    def ascii_dump(self):
        r"""
        Write an ASCII dump to stderr.

        Examples:

        >>> cmd  = 'from ppl import Constraint_System, Variable\n'
        >>> cmd += 'x = Variable(0)\n'
        >>> cmd += 'y = Variable(1)\n'
        >>> cmd += 'cs = Constraint_System( 3*x > 2*y+1 )\n'
        >>> cmd += 'cs.ascii_dump()\n'
        >>> import subprocess, sys
        >>> proc = subprocess.Popen([sys.executable, '-c', cmd], stderr=subprocess.PIPE)
        >>> out, err = proc.communicate()
        >>> print(str(err.decode('ascii')))
        topology NOT_NECESSARILY_CLOSED
        ...
        <BLANKLINE>
        """
        self.thisptr.ascii_dump()

    def __len__(self):
        """
        Return the number of constraints in the system.

        Examples:

        >>> from ppl import Variable, Constraint_System
        >>> x = Variable(0)
        >>> cs = Constraint_System( [x>0, x<1] )
        >>> len(cs)
        2
        >>> cs.insert(2*x < 3)
        >>> len(cs)
        3
        >>> cs.clear()
        >>> len(cs)
        0
        """
        cdef Py_ssize_t length = 0
        cdef PPL_Constraint_System_iterator *csi_ptr = new PPL_Constraint_System_iterator(self.thisptr[0].begin())
        while csi_ptr[0] != self.thisptr[0].end():
            length += 1
            csi_ptr[0].inc(1)
        del csi_ptr
        return length

    def __iter__(self):
        """
        Iterate through the constraints of the system.

        Examples:

        >>> from ppl import Variable, Constraint_System
        >>> x = Variable(0)
        >>> cs = Constraint_System( x>0 )
        >>> iter = cs.__iter__()
        >>> next(iter)
        x0>0
        >>> list(cs)   # uses __iter__() internally
        [x0>0]
        >>> x = Variable(0)
        >>> y = Variable(1)
        >>> cs = Constraint_System( 5*x < 2*y )
        >>> cs.insert( 6*x-y == 0 )
        >>> cs.insert( x >= 2*x-7*y )
        >>> next(cs.__iter__())
        -5*x0+2*x1>0
        >>> list(cs)
        [-5*x0+2*x1>0, 6*x0-x1==0, -x0+7*x1>=0]
        >>> x = Variable(0)
        >>> cs = Constraint_System( x > 0 )
        >>> cs.insert ( 2*x <= -3)
        >>> it = cs.__iter__()
        >>> next(it)
        x0>0
        >>> next(it)
        -2*x0-3>=0
        >>> next(it)
        Traceback (most recent call last):
        ...
        StopIteration
        """
        cdef PPL_Constraint_System_iterator *csi_ptr = new PPL_Constraint_System_iterator(self.thisptr[0].begin())
        try:
            while csi_ptr[0] != self.thisptr[0].end():
                yield _wrap_Constraint(deref(csi_ptr[0].inc(1)))
        finally:
            del csi_ptr

    def __getitem__(self, int k):
        """
        Return the k-th constraint.

        The correct way to read the individual constraints is to
        iterate over the constraint system. This method is for
        convenience only.

        INPUT:

        - ``k`` -- integer. The index of the constraint.

        OUTPUT:

        The `k`-th constraint of the constraint system.

        Examples:

        >>> from ppl import Variable, Constraint_System
        >>> x = Variable(0)
        >>> cs = Constraint_System( x>0 )
        >>> cs.insert( x<1 )
        >>> cs
        Constraint_System {x0>0, -x0+1>0}
        >>> cs[0]
        x0>0
        >>> cs[1]
        -x0+1>0
        """
        if k < 0:
            raise IndexError('index must be nonnegative')
        iterator = iter(self)
        try:
            for i in range(k):
                next(iterator)
        except StopIteration:
            raise IndexError('index is past-the-end')
        return next(iterator)

    def __repr__(self):
        r"""
        Return a string representation of the constraint system.

        OUTPUT:

        A string.

        Examples:

        >>> from ppl import Constraint_System, Variable
        >>> x = Variable(0)
        >>> y = Variable(1)
        >>> cs = Constraint_System([3*x+2*y+1 < 3, 0*x>x+1])
        >>> cs.__repr__()
        'Constraint_System {-3*x0-2*x1+2>0, -x0-1>0}'
        """
        s = 'Constraint_System {'
        s += ', '.join([repr(c) for c in self])
        s += '}'
        return s

    def __reduce__(self):
        """
        Pickle object.

        Tests:

        >>> from ppl import Constraint_System, Variable
        >>> from pickle import loads, dumps
        >>> x = Variable(0)
        >>> y = Variable(1)
        >>> cs = Constraint_System([3*x+2*y+1 < 3, 0*x>x+1]);  cs
        Constraint_System {-3*x0-2*x1+2>0, -x0-1>0}
        >>> loads(dumps(cs))
        Constraint_System {-3*x0-2*x1+2>0, -x0-1>0}
        """
        return (Constraint_System, (tuple(self),))


cdef class Poly_Con_Relation(object):
    r"""
    Wrapper for PPL's ``Poly_Con_Relation`` class.

    INPUT/OUTPUT:

    You must not construct :class:`Poly_Con_Relation` objects
    manually. You will usually get them from
    :meth:`~sage.libs.ppl.Polyhedron.relation_with`. You can also get
    pre-defined relations from the class methods :meth:`nothing`,
    :meth:`is_disjoint`, :meth:`strictly_intersects`,
    :meth:`is_included`, and :meth:`saturates`.

    Examples:

    >>> from ppl import Poly_Con_Relation
    >>> saturates     = Poly_Con_Relation.saturates();  saturates
    saturates
    >>> is_included   = Poly_Con_Relation.is_included(); is_included
    is_included
    >>> is_included.implies(saturates)
    False
    >>> saturates.implies(is_included)
    False
    >>> rels = []
    >>> rels.append(Poly_Con_Relation.nothing())
    >>> rels.append(Poly_Con_Relation.is_disjoint())
    >>> rels.append(Poly_Con_Relation.strictly_intersects())
    >>> rels.append(Poly_Con_Relation.is_included())
    >>> rels.append(Poly_Con_Relation.saturates())
    >>> rels
    [nothing, is_disjoint, strictly_intersects, is_included, saturates]
    >>> for i, rel_i in enumerate(rels):
    ...       s = ""
    ...       for j, rel_j in enumerate(rels):
    ...           s=s+str(int(rel_i.implies(rel_j))) + ' '
    ...       print(" ".join(str(int(rel_i.implies(rel_j))) for j, rel_j in enumerate(rels)))
    1 0 0 0 0
    1 1 0 0 0
    1 0 1 0 0
    1 0 0 1 0
    1 0 0 0 1
    """
    def __cinit__(self, do_not_construct_manually=False):
        """
        The Cython constructor.

        See :class:`Poly_Con_Relation` for documentation.

        Tests:

        >>> from ppl import Poly_Con_Relation
        >>> Poly_Con_Relation.nothing()
        nothing
        """
        assert(do_not_construct_manually)
        self.thisptr = NULL

    def __dealloc__(self):
        """
        The Cython destructor.
        """
        assert self.thisptr!=NULL, 'Do not construct Poly_Con_Relation objects manually!'
        del self.thisptr

    def __hash__(self):
        r"""
        Tests:

        >>> import ppl
        >>> hash(ppl.Poly_Con_Relation.saturates())
        Traceback (most recent call last):
        ...
        TypeError: Poly_Con_Relation unhashable
        """
        raise TypeError('Poly_Con_Relation unhashable')

    def implies(self, Poly_Con_Relation y):
        r"""
        Test whether ``self`` implies ``y``.

        INPUT:

        - ``y`` -- a :class:`Poly_Con_Relation`.

        OUTPUT:

        Boolean. ``True`` if and only if ``self`` implies ``y``.

        Examples:

            >>> from ppl import Poly_Con_Relation
            >>> nothing = Poly_Con_Relation.nothing()
            >>> nothing.implies( nothing )
            True
        """
        return self.thisptr.implies(y.thisptr[0])

    @classmethod
    def nothing(cls):
        r"""
        Return the assertion that says nothing.

        OUTPUT:

        A :class:`Poly_Con_Relation`.

        Examples:

        >>> from ppl import Poly_Con_Relation
        >>> Poly_Con_Relation.nothing()
        nothing
        """
        return _wrap_Poly_Con_Relation(PPL_Poly_Con_Relation_nothing())

    @classmethod
    def is_disjoint(cls):
        r"""
        Return the assertion "The polyhedron and the set of points
        satisfying the constraint are disjoint".

        OUTPUT:

        A :class:`Poly_Con_Relation`.

        Examples:

        >>> from ppl import Poly_Con_Relation
        >>> Poly_Con_Relation.is_disjoint()
        is_disjoint
        """
        return _wrap_Poly_Con_Relation(PPL_Poly_Con_Relation_is_disjoint())

    @classmethod
    def strictly_intersects(cls):
        r"""
        Return the assertion "The polyhedron intersects the set of
        points satisfying the constraint, but it is not included in
        it".

        :return: a :class:`Poly_Con_Relation`.

        Examples:

        >>> from ppl import Poly_Con_Relation
        >>> Poly_Con_Relation.strictly_intersects()
        strictly_intersects
        """
        return _wrap_Poly_Con_Relation(PPL_Poly_Con_Relation_strictly_intersects())

    @classmethod
    def is_included(cls):
        r"""
        Return the assertion "The polyhedron is included in the set of
        points satisfying the constraint".

        OUTPUT:

        A :class:`Poly_Con_Relation`.

        Examples:

        >>> from ppl import Poly_Con_Relation
        >>> Poly_Con_Relation.is_included()
        is_included
        """
        return _wrap_Poly_Con_Relation(PPL_Poly_Con_Relation_is_included())

    @classmethod
    def saturates(cls):
        r"""
        Return the assertion "".

        OUTPUT:

        A :class:`Poly_Con_Relation`.

        Examples:

        >>> from ppl import Poly_Con_Relation
        >>> Poly_Con_Relation.saturates()
        saturates
        """
        return _wrap_Poly_Con_Relation(PPL_Poly_Con_Relation_saturates())

    def ascii_dump(self):
        r"""
        Write an ASCII dump to stderr.

        Examples:

        >>> cmd  = 'from ppl import Poly_Con_Relation\n'
        >>> cmd += 'Poly_Con_Relation.nothing().ascii_dump()\n'
        >>> import subprocess, sys
        >>> proc = subprocess.Popen([sys.executable, '-c', cmd], stderr=subprocess.PIPE)
        >>> out, err = proc.communicate()
        >>> print(str(err.decode('ascii')))
        NOTHING
        """
        self.thisptr.ascii_dump()

    def __repr__(self):
        r"""
        Return a string representation.

        OUTPUT:

        String.

        Examples:

        >>> from ppl import Poly_Con_Relation
        >>> Poly_Con_Relation.nothing().__repr__()
        'nothing'
        """
        rel = []
        if self.implies(Poly_Con_Relation.is_disjoint()):
            rel.append('is_disjoint')
        if self.implies(Poly_Con_Relation.strictly_intersects()):
            rel.append('strictly_intersects')
        if self.implies(Poly_Con_Relation.is_included()):
            rel.append('is_included')
        if self.implies(Poly_Con_Relation.saturates()):
            rel.append('saturates')
        if rel:
            return ', '.join(rel)
        else:
            return 'nothing'


cdef _make_Constraint_from_richcmp(lhs_, rhs_, op):
    cdef Linear_Expression lhs = Linear_Expression(lhs_)
    cdef Linear_Expression rhs = Linear_Expression(rhs_)
    if op == Py_LT:
        return _wrap_Constraint(lhs.thisptr[0] < rhs.thisptr[0])
    elif op == Py_LE:
        return _wrap_Constraint(lhs.thisptr[0] <= rhs.thisptr[0])
    elif op == Py_EQ:
        return _wrap_Constraint(lhs.thisptr[0] == rhs.thisptr[0])
    elif op == Py_GT:
        return _wrap_Constraint(lhs.thisptr[0] > rhs.thisptr[0])
    elif op == Py_GE:
        return _wrap_Constraint(lhs.thisptr[0] >= rhs.thisptr[0])
    elif op == Py_NE:
        raise NotImplementedError
    else:
        assert(False)


cdef _wrap_Constraint(PPL_Constraint constraint):
    """
    Wrap a C++ ``PPL_Constraint`` into a Cython ``Constraint``.
    """
    cdef Constraint c = Constraint.__new__(Constraint)
    c.thisptr = new PPL_Constraint(constraint)
    return c


cdef _wrap_Constraint_System(PPL_Constraint_System constraint_system):
    """
    Wrap a C++ ``PPL_Constraint_System`` into a Cython ``Constraint_System``.
    """
    cdef Constraint_System cs = Constraint_System.__new__(Constraint_System)
    cs.thisptr = new PPL_Constraint_System(constraint_system)
    return cs


cdef _wrap_Poly_Con_Relation(PPL_Poly_Con_Relation relation):
    """
    Wrap a C++ ``PPL_Poly_Con_Relation`` into a Cython ``Poly_Con_Relation``.
    """
    cdef Poly_Con_Relation rel = Poly_Con_Relation(True)
    rel.thisptr = new PPL_Poly_Con_Relation(relation)
    return rel



# distutils: language = c++
# distutils: libraries = gmp gmpxx ppl m
#*****************************************************************************
#       Copyright (C) 2020 Vincent Delecroix <vincent.delecroix@labri.fr>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#  as published by the Free Software Foundation; either version 3 of
#  the License, or (at youroption) any later version.
#                  http://www.gnu.org/licenses/
#*****************************************************************************

from cython.operator cimport dereference as deref

from gmpy2 cimport GMPy_MPZ_From_mpz, mpz, import_gmpy2

from .constraint cimport Constraint
from .linear_algebra cimport Variable, Linear_Expression, PPL_Coefficient_from_pyobject

import_gmpy2()


def _dummy():
    raise ValueError


cdef class Congruence(object):
    r"""
    Wrapper for PPL's ``Congruence`` class.

    >>> import ppl
    >>> x = ppl.Variable(0)
    >>> y = ppl.Variable(1)
    >>> ppl.Congruence(x + 2*y - 1, 7)
    x0+2*x1-1==0 (mod 7)
    >>> ppl.Congruence(x + y == 2, 5)
    x0+x1-2==0 (mod 5)
    """
    def __init__(self, arg=None, mod=None):
        if arg is None:
            self.thisptr = new PPL_Congruence()
        elif isinstance(arg, Congruence):
            self.thisptr = new PPL_Congruence((<Congruence> arg).thisptr[0])
        elif isinstance(arg, Variable):
            self.thisptr = new PPL_Congruence()
            if mod is None:
                mod = mpz(1)
            self.thisptr[0] = modulo(PPL_Linear_Expression((<Variable> arg).thisptr[0]),
                                     PPL_Coefficient_from_pyobject(mod))
        elif isinstance(arg, Linear_Expression):
            self.thisptr = new PPL_Congruence()
            if mod is None:
                mod = mpz(1)
            self.thisptr[0] = modulo((<Linear_Expression> arg).thisptr[0],
                                     PPL_Coefficient_from_pyobject(mod))
        elif isinstance(arg, Constraint):
            self.thisptr = new PPL_Congruence((<Constraint> arg).thisptr[0])
            if mod is None:
                mod = mpz(1)
            self.thisptr.set_modulus(PPL_Coefficient_from_pyobject(mod))
        else:
            raise TypeError("invalid argument for Congruence")

    def __cinit__(self):
        self.thisptr = NULL

    def __dealloc__(self):
        del self.thisptr

    def is_equal(self, Congruence other):
        r"""
        Return whether ``self`` is equal to ``other``.

        Examples:

        >>> import ppl
        >>> x = ppl.Variable(0)
        >>> Congruence(x + 1, 3).is_equal(Congruence(x + 1, 3))
        True
        >>> Congruence(x, 3).is_equal(Congruence(x - 2, 3))
        False
        >>> Congruence(x, 3).is_equal(Congruence(x , 2))
        False
        """
        return (<Congruence> self).thisptr[0] == (<Congruence> other).thisptr[0]

    def coefficient(self, v):
        r"""
        Return the coefficient ``v`` of this congruence.

        Examples:

        >>> import ppl
        >>> x = ppl.Variable(0)
        >>> y = ppl.Variable(1)
        >>> c = (2 * x + y == 1) % 12
        >>> c.coefficient(0)
        mpz(2)
        >>> c.coefficient(x)
        mpz(2)

        Note that contrarily to :class:`Linear_Expression` the congruences raise
        an error when trying to access a coefficient beyond the dimension

        >>> c.coefficient(ppl.Variable(13))
        Traceback (most recent call last):
        ...
        ValueError: PPL::Congruence::coefficient(v):
        this->space_dimension() == 2, v.space_dimension() == 14.
        """
        cdef Variable vv
        if type(v) is Variable:
            vv = <Variable> v
        else:
            vv = Variable(v)
        return GMPy_MPZ_From_mpz(self.thisptr.coefficient(vv.thisptr[0]).get_mpz_t())

    def coefficients(self):
        r"""
        Return the coefficients of this congruence as a tuple.

        Examples:

        >>> import ppl
        >>> x = ppl.Variable(0)
        >>> y = ppl.Variable(1)
        >>> t = ppl.Variable(3)
        >>> c = ppl.Congruence(x + 2*y + t - 1, 7)
        >>> c.coefficients()
        (mpz(1), mpz(2), mpz(0), mpz(1))
        """
        cdef int d = self.space_dimension()
        cdef int i
        coeffs = []
        for i in range(0, d):
            coeffs.append(GMPy_MPZ_From_mpz(self.thisptr.coefficient(PPL_Variable(i)).get_mpz_t()))
        return tuple(coeffs)

    def modulus(self):
        r"""
        Return the modulus of this congruence.

        Examples:

        >>> import ppl
        >>> x = ppl.Variable(0)
        >>> y = ppl.Variable(1)
        >>> c = ppl.Congruence(x + 2*y - 3, 7)
        >>> c.modulus()
        mpz(7)
        """
        return GMPy_MPZ_From_mpz(self.thisptr.modulus().get_mpz_t())

    def inhomogeneous_term(self):
        r"""
        Return the inhomogeneous term of this congruence.

        Examples:

        >>> import ppl
        >>> x = ppl.Variable(0)
        >>> y = ppl.Variable(1)
        >>> c = ppl.Congruence(x + 2*y - 3, 7)
        >>> c.inhomogeneous_term()
        mpz(-3)
        """
        return GMPy_MPZ_From_mpz(self.thisptr.inhomogeneous_term().get_mpz_t())

    def space_dimension(self):
        return self.thisptr.space_dimension()

    def __repr__(self):
        s = ''
        v = [Variable(i) for i in range(self.space_dimension())]
        e = sum(self.coefficient(x)*x for x in v)
        e += self.inhomogeneous_term()
        s = repr(e) + '==0 (mod %s)' % self.modulus()
        return s

    def __reduce__(self):
        r"""
        >>> from ppl import Variable, Congruence, Congruence_System
        >>> from pickle import loads, dumps
        >>> x = Variable(0)
        >>> y = Variable(1)
        >>> z = Variable(2)
        >>> c1 = Congruence(2*x + 3*y - 5*z == 3, 12)
        >>> c1
        2*x0+3*x1-5*x2-3==0 (mod 12)
        >>> loads(dumps(c1))
        2*x0+3*x1-5*x2-3==0 (mod 12)
        >>> assert c1.is_equal(loads(dumps(c1)))
        """
        le = Linear_Expression(self.coefficients(), self.inhomogeneous_term())
        return (congruence, (le == 0, self.modulus()))

cdef class Congruence_System(object):
    r"""
    Wrapper for PPL's ``Congruence_System`` class.

    >>> from ppl import Variable, Congruence, Congruence_System
    >>> x = Variable(0)
    >>> y = Variable(1)
    >>> z = Variable(2)
    >>> c1 = Congruence(2*x + 3*y - 5*z == 0, 12)
    >>> c2 = Congruence(4*x + y == 5, 18)
    >>> C = Congruence_System()
    >>> C.insert(c1)
    >>> C.insert(c2)
    >>> C
    Congruence_System {2*x0+3*x1-5*x2==0 (mod 12), 4*x0+x1+13==0 (mod 18)}
    """
    def __init__(self, arg=None):
        if arg is None:
            self.thisptr = new PPL_Congruence_System()
        elif isinstance(arg, Congruence):
            c = <Congruence> arg
            self.thisptr = new PPL_Congruence_System(c.thisptr[0])
        elif isinstance(arg, Congruence_System):
            cs = <Congruence_System> arg
            self.thisptr = new PPL_Congruence_System(cs.thisptr[0])
        elif isinstance(arg, (list, tuple)):
            self.thisptr = new PPL_Congruence_System()
            for congruence in arg:
                self.insert(congruence)

    def ascii_dump(self):
        r"""
        Write an ASCII dump of this congruence to stderr.

        Examples:

        >>> cmd  = 'from ppl import Variable, Congruence_System\n'
        >>> cmd += 'x = Variable(0)\n'
        >>> cmd += 'y = Variable(1)\n'
        >>> cmd += 'cs = Congruence_System( (3*x == 2*y+1) % 7 )\n'
        >>> cmd += 'cs.ascii_dump()\n'
        >>> import subprocess, sys
        >>> proc = subprocess.Popen([sys.executable, '-c', cmd], stderr=subprocess.PIPE)
        >>> out, err = proc.communicate()
        >>> print(str(err.decode('ascii')))
        1 x 2 SPARSE
        size 3 6 3 -2 m 7
        <BLANKLINE>
        """
        self.thisptr.ascii_dump()

    def __len__(self):
        r"""
        Return the number of congruences in the system.

        Examples:

        >>> from ppl import Variable, Congruence_System
        >>> x = Variable(0)
        >>> y = Variable(1)
        >>> cs = Congruence_System( [(x == 3) % 15, (2*y == 7) % 12])
        >>> len(cs)
        2
        >>> cs.insert((2*x + y == 12) % 22)
        >>> len(cs)
        3
        >>> cs.clear()
        >>> len(cs)
        0
        """
        cdef Py_ssize_t l = 0
        cdef PPL_Congruence_System_iterator * csi_ptr = new PPL_Congruence_System_iterator(self.thisptr[0].begin())

        while csi_ptr[0] != self.thisptr[0].end():
            l += 1
            csi_ptr[0].inc(1)
        del csi_ptr
        return l

    def __iter__(self):
        cdef PPL_Congruence_System_iterator * csi_ptr = new PPL_Congruence_System_iterator(self.thisptr[0].begin())

        try:
            while csi_ptr[0] != self.thisptr[0].end():
                yield _wrap_Congruence(deref(csi_ptr[0].inc(1)))
        finally:
            del csi_ptr

    def __repr__(self):
        s = 'Congruence_System {'
        s += ', '.join([repr(c) for c in self])
        s += '}'
        return s

    def insert(self, Congruence c):
        r"""
        Insert the congruence ``c`` into the congruence system.

        Examples:

        >>> from ppl import Variable, Congruence_System
        >>> x = Variable(0)
        >>> y = Variable(1)
        >>> cs = Congruence_System()
        >>> cs.insert((3 * x + 2 == 0) % 13)
        >>> cs
        Congruence_System {3*x0+2==0 (mod 13)}
        """
        self.thisptr.insert(c.thisptr[0])

    def clear(self):
        r"""
        Remove all congruences from this congruence system.

        >>> from ppl import Variable, Congruence_System
        >>> x = Variable(0)
        >>> y = Variable(1)
        >>> cs = Congruence_System([(3*x+2*y+1 == 3) % 15, (x+2*y + 7 == 0) % 22])
        >>> len(cs)
        2
        >>> cs.clear()
        >>> len(cs)
        0
        """
        self.thisptr.clear()

    def __reduce__(self):
        """
        Pickle object.

        Examples:

        >>> from ppl import Variable, Congruence_System
        >>> from pickle import loads, dumps
        >>> x = Variable(0)
        >>> y = Variable(1)
        >>> cs = Congruence_System([(3*x+2*y+1 == 3) % 15, (x+2*y + 7 == 0) % 22])
        >>> cs
        Congruence_System {3*x0+2*x1+13==0 (mod 15), x0+2*x1+7==0 (mod 22)}
        >>> loads(dumps(cs))
        Congruence_System {3*x0+2*x1+13==0 (mod 15), x0+2*x1+7==0 (mod 22)}
        """
        return (Congruence_System, (tuple(self),))


cdef _wrap_Congruence(PPL_Congruence congruence):
    cdef Congruence c = Congruence.__new__(Congruence)
    c.thisptr = new PPL_Congruence(congruence)
    return c


def congruence(le, m):
    return Congruence(le, m)



# distutils: language = c++
# distutils: libraries = gmp gmpxx ppl m
#*****************************************************************************
#       Copyright (C) 2010-2014 Volker Braun  <vbraun.name@gmail.com>
#                     2011 Simon King <simon.king@uni-jena.de>
#                     2011-2017 Jeroen Demeyer <jdemeyer@cage.ugent.be>
#                     2012 Risan <ptrrsn.1@gmail.com>
#                     2013 Julien Puydt <julien.puydt@laposte.net>
#                     2013 Travis Scrimshaw <tscrim@ucdavis.edu>
#                     2015 André Apitzsch <andre.apitzsch@etit.tu-chemnitz.de>
#                     2016 Jori Mäntysalo <jori.mantysalo@uta.fi>
#                     2016 Matthias Koeppe <mkoeppe@math.ucdavis.edu>
#                     2016-2017 Frédéric Chapoton <chapoton@math.univ-lyon1.fr>
#                     2016-2018 Vincent Delecroix <vincent.delecroix@labri.fr>
#                     2017-2018 Vincent Klein <vinklein@gmail.com>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#  as published by the Free Software Foundation; either version 3 of
#  the License, or (at youroption) any later version.
#                  http://www.gnu.org/licenses/
#*****************************************************************************

from gmpy2 cimport GMPy_MPZ_From_mpz, import_gmpy2
from cython.operator cimport dereference as deref
from .linear_algebra cimport PPL_Coefficient_from_pyobject


# PPL can use floating-point arithmetic to compute integers
cdef extern from "ppl.hh" namespace "Parma_Polyhedra_Library":
    cdef void set_rounding_for_PPL()
    cdef void restore_pre_PPL_rounding()

# initialize gmpy2 C API
import_gmpy2()

# but with PPL's rounding the gsl will be very unhappy; must turn off!
restore_pre_PPL_rounding()

####################################################
cdef class Generator(object):
    r"""
    Wrapper for PPL's ``Generator`` class.

    An object of the class Generator is one of the following:

      * a line :math:`\ell = (a_0, \dots, a_{n-1})^T`

      * a ray :math:`r = (a_0, \dots, a_{n-1})^T`

      * a point :math:`p = (\tfrac{a_0}{d}, \dots, \tfrac{a_{n-1}}{d})^T`

      * a closure point :math:`c = (\tfrac{a_0}{d}, \dots, \tfrac{a_{n-1}}{d})^T`

    where :math:`n` is the dimension of the space and, for points and
    closure points, :math:`d` is the divisor.

    INPUT/OUTPUT:

    Use the helper functions :func:`line`, :func:`ray`, :func:`point`,
    and :func:`closure_point` to construct generators. Analogous class
    methods are also available, see :meth:`Generator.line`,
    :meth:`Generator.ray`, :meth:`Generator.point`,
    :meth:`Generator.closure_point`. Do not attempt to construct
    generators manually.

    .. NOTE::

        The generators are constructed from linear expressions. The
        inhomogeneous term is always silently discarded.

    Examples:

    >>> from ppl import Generator, Variable
    >>> x = Variable(0)
    >>> y = Variable(1)
    >>> Generator.line(5*x-2*y)
    line(5, -2)
    >>> Generator.ray(5*x-2*y)
    ray(5, -2)
    >>> Generator.point(5*x-2*y, 7)
    point(5/7, -2/7)
    >>> Generator.closure_point(5*x-2*y, 7)
    closure_point(5/7, -2/7)
    """
    def __cinit__(self, do_not_construct_manually=False):
        """
        The Cython constructor.

        See :class:`Variable` for documentation.

        Tests:

        >>> from ppl import Variable, line
        >>> x = Variable(0)
        >>> line(x)   # indirect doctest
        line(1)
        """
        assert(do_not_construct_manually)
        self.thisptr = NULL

    def __dealloc__(self):
        """
        The Cython destructor.
        """
        assert self.thisptr!=NULL, 'Do not construct Generators manually!'
        del self.thisptr

    def __hash__(self):
        r"""
        Tests:

        >>> import ppl
        >>> hash(ppl.point())
        Traceback (most recent call last):
        ...
        TypeError: Generator unhashable
        """
        raise TypeError('Generator unhashable')

    @classmethod
    def line(cls, expression):
        """
        Construct a line.

        INPUT:

        - ``expression`` -- a :class:`Linear_Expression` or something
          convertible to it (:class:`Variable` or integer).

        OUTPUT:

        A new :class:`Generator` representing the line.

        Raises a ``ValueError` if the homogeneous part of
        ``expression`` represents the origin of the vector space.

        Examples:

        >>> from ppl import Generator, Variable
        >>> y = Variable(1)
        >>> Generator.line(2*y)
        line(0, 1)
        >>> Generator.line(y)
        line(0, 1)
        >>> Generator.line(1)
        Traceback (most recent call last):
        ...
        ValueError: PPL::line(e):
        e == 0, but the origin cannot be a line.
        """
        cdef Linear_Expression e = Linear_Expression(expression)
        # This does not work as Cython gets confused by the private default ctor
        #   return _wrap_Generator(PPL_line(e.thisptr[0]))
        # workaround follows
        cdef Generator g = Generator(True)
        try:
            g.thisptr = new PPL_Generator(PPL_line(e.thisptr[0]))
        except BaseException:
            # g.thisptr must be set to something valid or g.__dealloc__() will segfault
            g.thisptr = new PPL_Generator(PPL_point(e.thisptr[0], PPL_Coefficient(1)))
            raise
        return g

    @classmethod
    def ray(cls, expression):
        """
        Construct a ray.

        INPUT:

        - ``expression`` -- a :class:`Linear_Expression` or something
          convertible to it (:class:`Variable` or integer).

        OUTPUT:

        A new :class:`Generator` representing the ray.

        Raises a ``ValueError` if the homogeneous part of
        ``expression`` represents the origin of the vector space.

        Examples:

        >>> from ppl import Generator, Variable
        >>> y = Variable(1)
        >>> Generator.ray(2*y)
        ray(0, 1)
        >>> Generator.ray(y)
        ray(0, 1)
        >>> Generator.ray(1)
        Traceback (most recent call last):
        ...
        ValueError: PPL::ray(e):
        e == 0, but the origin cannot be a ray.
        """
        cdef Linear_Expression e = Linear_Expression(expression)
        # This does not work as Cython gets confused by the private default ctor
        #   return _wrap_Generator(PPL_ray(e.thisptr[0]))
        # workaround follows
        cdef Generator g = Generator(True)
        try:
            g.thisptr = new PPL_Generator(PPL_ray(e.thisptr[0]))
        except BaseException:
            # g.thisptr must be set to something valid or g.__dealloc__() will segfault
            g.thisptr = new PPL_Generator(PPL_point(e.thisptr[0], PPL_Coefficient(1)))
            raise
        return g

    @classmethod
    def point(cls, expression=0, divisor=1):
        """
        Construct a point.

        INPUT:

        - ``expression`` -- a :class:`Linear_Expression` or something
          convertible to it (:class:`Variable` or integer).

        - ``divisor`` -- an integer.

        OUTPUT:

        A new :class:`Generator` representing the point.

        Raises a ``ValueError` if ``divisor==0``.

        Examples:

        >>> from ppl import Generator, Variable
        >>> y = Variable(1)
        >>> Generator.point(2*y+7, 3)
        point(0/3, 2/3)
        >>> Generator.point(y+7, 3)
        point(0/3, 1/3)
        >>> Generator.point(7, 3)
        point()
        >>> Generator.point(0, 0)
        Traceback (most recent call last):
        ...
        ValueError: PPL::point(e, d):
        d == 0.
        """
        cdef Linear_Expression e = Linear_Expression(expression)
        # This does not work as Cython gets confused by the private default ctor
        #   return _wrap_Generator(PPL_point(e.thisptr[0], PPL_Coefficient(d.value)))
        # workaround follows
        cdef Generator g = Generator(True)
        try:
            g.thisptr = new PPL_Generator(PPL_point(e.thisptr[0], PPL_Coefficient_from_pyobject(divisor)))
        except BaseException:
            # g.thisptr must be set to something valid or g.__dealloc__() will segfault
            g.thisptr = new PPL_Generator(PPL_point(e.thisptr[0], PPL_Coefficient(1)))
            raise
        return g

    @classmethod
    def closure_point(cls, expression=0, divisor=1):
        """
        Construct a closure point.

        A closure point is a point of the topological closure of a
        polyhedron that is not a point of the polyhedron itself.

        INPUT:

        - ``expression`` -- a :class:`Linear_Expression` or something
          convertible to it (:class:`Variable` or integer).

        - ``divisor`` -- an integer.

        OUTPUT:

        A new :class:`Generator` representing the point.

        Raises a ``ValueError` if ``divisor==0``.

        Examples:

        >>> from ppl import Generator, Variable
        >>> y = Variable(1)
        >>> Generator.closure_point(2*y+7, 3)
        closure_point(0/3, 2/3)
        >>> Generator.closure_point(y+7, 3)
        closure_point(0/3, 1/3)
        >>> Generator.closure_point(7, 3)
        closure_point()
        >>> Generator.closure_point(0, 0)
        Traceback (most recent call last):
        ...
        ValueError: PPL::closure_point(e, d):
        d == 0.
        """
        cdef Linear_Expression e = Linear_Expression(expression)
        # This does not work as Cython gets confused by the private default ctor
        #   return _wrap_Generator(PPL_closure_point(e.thisptr[0], PPL_Coefficient(d.value)))
        # workaround follows
        cdef Generator g = Generator(True)
        try:
            g.thisptr = new PPL_Generator(PPL_closure_point(e.thisptr[0], PPL_Coefficient_from_pyobject(divisor)))
        except BaseException:
            # g.thisptr must be set to something valid or g.__dealloc__() will segfault
            g.thisptr = new PPL_Generator(PPL_point(e.thisptr[0], PPL_Coefficient(1)))
            raise
        return g

    def __repr__(self):
        """
        Return a string representation of the generator.

        OUTPUT:

        String.

        Examples:

        >>> from ppl import Generator, Variable
        >>> x = Variable(0)
        >>> y = Variable(1)
        >>> e = 2*x-y+5
        >>> Generator.line(e)
        line(2, -1)
        >>> Generator.ray(e)
        ray(2, -1)
        >>> Generator.point(e, 3)
        point(2/3, -1/3)
        >>> Generator.closure_point(e, 3)
        closure_point(2/3, -1/3)
        """
        cdef PPL_GeneratorType t = self.thisptr.type()
        if t == LINE or t == RAY:
            div = ''
        elif t == POINT or t == CLOSURE_POINT:
            div = '/' + str(self.divisor())
        else:
            raise RuntimeError

        s = ', '.join(str(self.coefficient(Variable(i))) + div for i in range(self.space_dimension()))
        return PPL_GeneratorType_str(t) + '(' + s + ')'

    def space_dimension(self):
        r"""
        Return the dimension of the vector space enclosing ``self``.

        OUTPUT:

        Integer.

        Examples:

        >>> from ppl import Variable, point
        >>> x = Variable(0)
        >>> y = Variable(1)
        >>> point(x).space_dimension()
        1
        >>> point(y).space_dimension()
        2
        """
        return self.thisptr.space_dimension()

    def set_space_dimension(self, size_t n):
        r"""
        Set the dimension of this generator to ``n``

        Examples:

        >>> import ppl
        >>> p = ppl.point()
        >>> p
        point()
        >>> p.set_space_dimension(5)
        >>> p
        point(0/1, 0/1, 0/1, 0/1, 0/1)

        >>> p = ppl.point(1 * ppl.Variable(0) + 2 * ppl.Variable(1) + 3 * ppl.Variable(2))
        >>> p.set_space_dimension(2)
        >>> p
        point(1/1, 2/1)
        """
        self.thisptr.set_space_dimension(n)

    def type(self):
        r"""
        Return the generator type of ``self``.

        OUTPUT:

        String. One of ``'line'``, ``'ray'``, ``'point'``, or
        ``'closure_point'``.

        Examples:

        >>> from ppl import Variable, point, closure_point, ray, line
        >>> x = Variable(0)
        >>> line(x).type()
        'line'
        >>> ray(x).type()
        'ray'
        >>> point(x,2).type()
        'point'
        >>> closure_point(x,2).type()
        'closure_point'
        """
        return PPL_GeneratorType_str(self.thisptr.type())

    def is_line(self):
        r"""
        Test whether ``self`` is a line.

        OUTPUT:

        Boolean.

        Examples:

        >>> from ppl import Variable, point, closure_point, ray, line
        >>> x = Variable(0)
        >>> line(x).is_line()
        True
        >>> ray(x).is_line()
        False
        >>> point(x,2).is_line()
        False
        >>> closure_point(x,2).is_line()
        False
        """
        return self.thisptr.is_line()

    def is_ray(self):
        r"""
        Test whether ``self`` is a ray.

        OUTPUT:

        Boolean.

        Examples:

        >>> from ppl import Variable, point, closure_point, ray, line
        >>> x = Variable(0)
        >>> line(x).is_ray()
        False
        >>> ray(x).is_ray()
        True
        >>> point(x,2).is_ray()
        False
        >>> closure_point(x,2).is_ray()
        False
        """
        return self.thisptr.is_ray()

    def is_line_or_ray(self):
        r"""
        Test whether ``self`` is a line or a ray.

        OUTPUT:

        Boolean.

        Examples:

        >>> from ppl import Variable, point, closure_point, ray, line
        >>> x = Variable(0)
        >>> line(x).is_line_or_ray()
        True
        >>> ray(x).is_line_or_ray()
        True
        >>> point(x,2).is_line_or_ray()
        False
        >>> closure_point(x,2).is_line_or_ray()
        False
        """
        return self.thisptr.is_line_or_ray()

    def is_point(self):
        r"""
        Test whether ``self`` is a point.

        OUTPUT:

        Boolean.

        Examples:

        >>> from ppl import Variable, point, closure_point, ray, line
        >>> x = Variable(0)
        >>> line(x).is_point()
        False
        >>> ray(x).is_point()
        False
        >>> point(x,2).is_point()
        True
        >>> closure_point(x,2).is_point()
        False
        """
        return self.thisptr.is_point()

    def is_closure_point(self):
        r"""
        Test whether ``self`` is a closure point.

        OUTPUT:

        Boolean.

        Examples:

        >>> from ppl import Variable, point, closure_point, ray, line
        >>> x = Variable(0)
        >>> line(x).is_closure_point()
        False
        >>> ray(x).is_closure_point()
        False
        >>> point(x,2).is_closure_point()
        False
        >>> closure_point(x,2).is_closure_point()
        True
        """
        return self.thisptr.is_closure_point()

    def coefficient(self, Variable v):
        """
        Return the coefficient of the variable ``v``.

        INPUT:

        - ``v`` -- a :class:`Variable`.

        OUTPUT:

        An integer.

        Examples:

        >>> from ppl import Variable, line
        >>> x = Variable(0)
        >>> line = line(3*x+1)
        >>> line
        line(1)
        >>> line.coefficient(x)
        mpz(1)
        """
        return GMPy_MPZ_From_mpz(self.thisptr.coefficient(v.thisptr[0]).get_mpz_t())

    def coefficients(self):
        """
        Return the coefficients of the generator.

        See also :meth:`coefficient`.

        OUTPUT:

        A tuple of integers of length :meth:`space_dimension`.

        Examples:

            >>> from ppl import Variable, point
            >>> x = Variable(0);  y = Variable(1)
            >>> p = point(3*x+5*y+1, 2); p
            point(3/2, 5/2)
            >>> p.coefficients()
            (mpz(3), mpz(5))
        """
        cdef int d = self.space_dimension()
        cdef int i
        coeffs = []
        for i in range(0, d):
            coeffs.append(GMPy_MPZ_From_mpz(self.thisptr.coefficient(PPL_Variable(i)).get_mpz_t()))
        return tuple(coeffs)

    def divisor(self):
        """
        If ``self`` is either a point or a closure point, return its
        divisor.

        OUTPUT:

        An integer. If ``self`` is a ray or a line, raises
        ``ValueError``.

        Examples:

        >>> from ppl import Generator, Variable
        >>> x = Variable(0)
        >>> y = Variable(1)
        >>> point = Generator.point(2*x-y+5)
        >>> point.divisor()
        mpz(1)
        >>> line = Generator.line(2*x-y+5)
        >>> line.divisor()
        Traceback (most recent call last):
        ...
        ValueError: PPL::Generator::divisor():
        *this is neither a point nor a closure point.
        """
        return GMPy_MPZ_From_mpz(self.thisptr.divisor().get_mpz_t())

    def is_equivalent_to(self, Generator g):
        r"""
        Test whether ``self`` and ``g`` are equivalent.

        INPUT:

        - ``g`` -- a :class:`Generator`.

        OUTPUT:

        Boolean. Returns ``True`` if and only if ``self`` and ``g``
        are equivalent generators.

        Note that generators having different space dimensions are not
        equivalent.

        Examples:

        >>> from ppl import Variable, point, line
        >>> x = Variable(0)
        >>> y = Variable(1)
        >>> point(2*x    , 2).is_equivalent_to( point(x) )
        True
        >>> point(2*x+0*y, 2).is_equivalent_to( point(x) )
        False
        >>> line(4*x).is_equivalent_to(line(x))
        True
        """
        return self.thisptr.is_equivalent_to(g.thisptr[0])

    def ascii_dump(self):
        r"""
        Write an ASCII dump to stderr.

        Examples:

        >>> cmd  = 'from ppl import Linear_Expression, Variable, point\n'
        >>> cmd += 'x = Variable(0)\n'
        >>> cmd += 'y = Variable(1)\n'
        >>> cmd += 'p = point(3*x+2*y)\n'
        >>> cmd += 'p.ascii_dump()\n'
        >>> import subprocess
        >>> import sys
        >>> proc = subprocess.Popen([sys.executable, '-c', cmd], stderr=subprocess.PIPE)
        >>> out, err = proc.communicate()
        >>> print(str(err.decode('ascii')))
        size 3 1 3 2 ...
        <BLANKLINE>
        """
        self.thisptr.ascii_dump()

    def __reduce__(self):
        """
        Pickle object.

        Tests:

        >>> from ppl import Generator, Variable
        >>> from pickle import loads, dumps
        >>> x = Variable(0); y = Variable(1)
        >>> loads(dumps(Generator.point(2*x+7*y, 3)))
        point(2/3, 7/3)
        >>> loads(dumps(Generator.closure_point(2*x+7*y, 3)))
        closure_point(2/3, 7/3)
        >>> loads(dumps(Generator.line(2*x+7*y)))
        line(2, 7)
        >>> loads(dumps(Generator.ray(2*x+7*y)))
        ray(2, 7)
        """
        le = Linear_Expression(self.coefficients(), 0)
        t = self.thisptr.type()
        if t == LINE:
            return (Generator.line, (le,))
        elif t == RAY:
            return (Generator.ray, (le,))
        elif t == POINT:
            return (Generator.point, (le, self.divisor()))
        elif t == CLOSURE_POINT:
            return (Generator.closure_point, (le, self.divisor()))
        else:
            raise RuntimeError

    def permute_space_dimensions(self, cycle):
        r"""
        Permute the coordinates according to ``cycle``.

        >>> from ppl import Generator, Variable
        >>> x = Variable(0); y = Variable(1); z = Variable(2)
        >>> p = Generator.point(2*x+7*y-z, 3)
        >>> p.permute_space_dimensions([0, 1])
        >>> p
        point(7/3, 2/3, -1/3)
        >>> p.permute_space_dimensions([0, 2, 1])
        >>> p
        point(2/3, -1/3, 7/3)
        """
        cdef cppvector[PPL_Variable] cpp_cycle
        cdef int i
        for i in cycle:
            cpp_cycle.push_back(PPL_Variable(i))
        self.thisptr.permute_space_dimensions(cpp_cycle)

####################################################
### Generator_System  ##############################
####################################################
####################################################
cdef class Generator_System(object):
    """
    Wrapper for PPL's ``Generator_System`` class.

    An object of the class Generator_System is a system of generators,
    i.e., a multiset of objects of the class Generator (lines, rays,
    points and closure points). When inserting generators in a system,
    space dimensions are automatically adjusted so that all the
    generators in the system are defined on the same vector space. A
    system of generators which is meant to define a non-empty
    polyhedron must include at least one point: the reason is that
    lines, rays and closure points need a supporting point (lines and
    rays only specify directions while closure points only specify
    points in the topological closure of the NNC polyhedron).

    Examples:

        >>> from ppl import Generator_System, Variable, line, ray, point, closure_point
        >>> x = Variable(0)
        >>> y = Variable(1)
        >>> gs = Generator_System(line(5*x-2*y))
        >>> gs.insert(ray(6*x-3*y))
        >>> gs.insert(point(2*x-7*y, 5))
        >>> gs.insert(closure_point(9*x-1*y, 2))
        >>> gs
        Generator_System {line(5, -2), ray(2, -1), point(2/5, -7/5), closure_point(9/2, -1/2)}
    """
    def __cinit__(self, arg=None):
        """
        The Cython constructor.

        See :class:`Generator_System` for documentation.

        Tests:

        >>> from ppl import Generator_System
        >>> Generator_System()   # indirect doctest
        Generator_System {}
        """
        if arg is None:
            self.thisptr = new PPL_Generator_System()
            return
        if isinstance(arg, Generator):
            g = <Generator>arg
            self.thisptr = new PPL_Generator_System(g.thisptr[0])
            return
        if isinstance(arg, Generator_System):
            gs = <Generator_System>arg
            self.thisptr = new PPL_Generator_System(gs.thisptr[0])
            return
        if isinstance(arg, (list, tuple)):
            self.thisptr = new PPL_Generator_System()
            for generator in arg:
                self.insert(generator)
            return
        raise ValueError('Cannot initialize with '+str(arg)+'.')

    def __dealloc__(self):
        """
        The Cython destructor.
        """
        del self.thisptr

    def __hash__(self):
        r"""
        Tests:

        >>> import ppl
        >>> hash(ppl.Generator_System())
        Traceback (most recent call last):
        ...
        TypeError: Generator_System unhashable
        """
        raise TypeError('Generator_System unhashable')

    def space_dimension(self):
        r"""
        Return the dimension of the vector space enclosing ``self``.

        OUTPUT:

        Integer.

        Examples:

        >>> from ppl import Variable, Generator_System, point
        >>> x = Variable(0)
        >>> gs = Generator_System( point(3*x) )
        >>> gs.space_dimension()
        1
        """
        return self.thisptr.space_dimension()

    def set_space_dimension(self, size_t n):
        r"""
        Set the dimension of the vector space enclosing ``self``

        Examples:

        >>> import ppl
        >>> gs = ppl.Generator_System()
        >>> gs.insert(ppl.point(ppl.Variable(0) + ppl.Variable(12)))
        >>> gs.insert(ppl.point(ppl.Variable(1) + 2*ppl.Variable(3)))
        >>> gs.space_dimension()
        13
        >>> gs.set_space_dimension(25)
        >>> gs.space_dimension()
        25
        >>> gs.set_space_dimension(3)
        >>> gs.space_dimension()
        3
        """
        self.thisptr.set_space_dimension(n)

    def clear(self):
        r"""
        Removes all generators from the generator system and sets its
        space dimension to 0.

        Examples:

        >>> from ppl import Variable, Generator_System, point
        >>> x = Variable(0)
        >>> gs = Generator_System( point(3*x) ); gs
        Generator_System {point(3/1)}
        >>> gs.clear()
        >>> gs
        Generator_System {}
        """
        self.thisptr.clear()

    def insert(self, Generator g):
        """
        Insert ``g`` into the generator system.

        The number of space dimensions of ``self`` is increased, if needed.

        INPUT:

        - ``g`` -- a :class:`Generator`.

        Examples:

        >>> from ppl import Variable, Generator_System, point
        >>> x = Variable(0)
        >>> gs = Generator_System( point(3*x) )
        >>> gs.insert( point(-3*x) )
        >>> gs
        Generator_System {point(3/1), point(-3/1)}
        """
        self.thisptr.insert(g.thisptr[0])

    def empty(self):
        """
        Return ``True`` if and only if ``self`` has no generators.

        OUTPUT:

        Boolean.

        Examples:

        >>> from ppl import Variable, Generator_System, point
        >>> x = Variable(0)
        >>> gs = Generator_System()
        >>> gs.empty()
        True
        >>> gs.insert( point(-3*x) )
        >>> gs.empty()
        False
        """
        return self.thisptr.empty()

    def ascii_dump(self):
        r"""
        Write an ASCII dump to stderr.

        Examples:

        >>> cmd  = 'from ppl import Generator_System, point, Variable\n'
        >>> cmd += 'x = Variable(0)\n'
        >>> cmd += 'y = Variable(1)\n'
        >>> cmd += 'gs = Generator_System( point(3*x+2*y+1) )\n'
        >>> cmd += 'gs.ascii_dump()\n'
        >>> import subprocess, sys
        >>> proc = subprocess.Popen([sys.executable, '-c', cmd], stderr=subprocess.PIPE)
        >>> out, err = proc.communicate()
        >>> print(str(err.decode('ascii')))
        topology NECESSARILY_CLOSED
        ...
        <BLANKLINE>
        """
        self.thisptr.ascii_dump()

    def __len__(self):
        """
        Return the number of generators in the system.

        >>> from ppl import Variable, Generator_System, point
        >>> x = Variable(0)
        >>> y = Variable(1)
        >>> gs = Generator_System()
        >>> gs.insert(point(3*x+2*y))
        >>> gs.insert(point(x))
        >>> gs.insert(point(y))
        >>> len(gs)
        3
        """
        return sum([1 for g in self])

    def __iter__(self):
        """
        Iterate through the generators of the system.
        Examples:

        >>> from ppl import Generator_System, Variable, point, ray, line, closure_point
        >>> x = Variable(0)
        >>> gs = Generator_System(point(3*x))
        >>> iter = gs.__iter__()
        >>> next(iter)
        point(3/1)
        >>> next(iter)
        Traceback (most recent call last):
        ...
        StopIteration
        >>> x = Variable(0)
        >>> y = Variable(1)
        >>> gs = Generator_System( line(5*x-2*y) )
        >>> gs.insert( ray(6*x-3*y) )
        >>> gs.insert( point(2*x-7*y, 5) )
        >>> gs.insert( closure_point(9*x-1*y, 2) )
        >>> next(gs.__iter__())
        line(5, -2)
        >>> list(gs)
        [line(5, -2), ray(2, -1), point(2/5, -7/5), closure_point(9/2, -1/2)]
        """
        cdef PPL_gs_iterator *gsi_ptr = new PPL_gs_iterator(self.thisptr[0].begin())
        try:
            while gsi_ptr[0] != self.thisptr[0].end():
                yield _wrap_Generator(deref(gsi_ptr[0].inc(1)))
        finally:
            del gsi_ptr

    def __getitem__(self, int k):
        """
        Return the ``k``-th generator.

        The correct way to read the individual generators is to
        iterate over the generator system. This method is for
        convenience only.

        INPUT:

        - ``k`` -- integer. The index of the generator.

        OUTPUT:

        The ``k``-th constraint of the generator system.

        Examples:

        >>> from ppl import Generator_System, Variable, point
        >>> x = Variable(0)
        >>> gs = Generator_System()
        >>> gs.insert(point(3*x))
        >>> gs.insert(point(-2*x))
        >>> gs
        Generator_System {point(3/1), point(-2/1)}
        >>> gs[0]
        point(3/1)
        >>> gs[1]
        point(-2/1)
        """
        if k < 0:
            raise IndexError('index must be nonnegative')
        iterator = iter(self)
        try:
            for i in range(k):
                next(iterator)
        except StopIteration:
            raise IndexError('index is past-the-end')
        return next(iterator)

    def __repr__(self):
        r"""
        Return a string representation of the generator system.

        OUTPUT:

        A string.

        Examples:

        >>> from ppl import Generator_System, Variable, point, ray
        >>> x = Variable(0)
        >>> y = Variable(1)
        >>> gs = Generator_System(point(3*x+2*y+1))
        >>> gs.insert(ray(x))
        >>> gs.__repr__()
        'Generator_System {point(3/1, 2/1), ray(1, 0)}'
        """
        s = 'Generator_System {'
        s += ', '.join([repr(g) for g in self])
        s += '}'
        return s

    def __reduce__(self):
        """
        Pickle object.

        Tests:

        >>> from ppl import Generator_System, Variable, point, ray
        >>> from pickle import loads, dumps
        >>> x = Variable(0)
        >>> y = Variable(1)
        >>> gs = Generator_System((point(3*x+2*y+1), ray(x)));  gs
        Generator_System {point(3/1, 2/1), ray(1, 0)}
        >>> loads(dumps(gs))
        Generator_System {point(3/1, 2/1), ray(1, 0)}
        """
        return (Generator_System, (tuple(self), ))

cdef PPL_GeneratorType_str(PPL_GeneratorType t):
    if t == LINE:
        return 'line'
    elif t == RAY:
        return 'ray'
    elif t == POINT:
        return 'point'
    elif t == CLOSURE_POINT:
        return 'closure_point'
    else:
        raise RuntimeError

cdef _wrap_Generator(PPL_Generator generator):
    """
    Wrap a C++ ``PPL_Generator`` into a Cython ``Generator``.
    """
    cdef Generator g = Generator(True)
    g.thisptr = new PPL_Generator(generator)
    return g


cdef _wrap_Generator_System(PPL_Generator_System generator_system):
    """
    Wrap a C++ ``PPL_Generator_System`` into a Cython ``Generator_System``.
    """
    cdef Generator_System gs = Generator_System()
    del gs.thisptr
    gs.thisptr = new PPL_Generator_System(generator_system)
    return gs

cdef class Poly_Gen_Relation(object):
    r"""
    Wrapper for PPL's ``Poly_Con_Relation`` class.

    INPUT/OUTPUT:

    You must not construct :class:`Poly_Gen_Relation` objects
    manually. You will usually get them from
    :meth:`~sage.libs.ppl.Polyhedron.relation_with`. You can also get
    pre-defined relations from the class methods :meth:`nothing` and
    :meth:`subsumes`.

    Examples:

    >>> from ppl import Poly_Gen_Relation
    >>> nothing = Poly_Gen_Relation.nothing(); nothing
    nothing
    >>> subsumes = Poly_Gen_Relation.subsumes(); subsumes
    subsumes
    >>> nothing.implies( subsumes )
    False
    >>> subsumes.implies( nothing )
    True
    """
    def __cinit__(self, do_not_construct_manually=False):
        """
        The Cython constructor.

        See :class:`Poly_Gen_Relation` for documentation.

        Tests:

        >>> from ppl import Poly_Gen_Relation
        >>> Poly_Gen_Relation.nothing()
        nothing
        """
        assert(do_not_construct_manually)
        self.thisptr = NULL

    def __dealloc__(self):
        """
        The Cython destructor.
        """
        assert self.thisptr!=NULL, 'Do not construct Poly_Gen_Relation objects manually!'
        del self.thisptr

    def __hash__(self):
        r"""
        Tests:

        >>> import ppl
        >>> hash(ppl.Poly_Gen_Relation.nothing())
        Traceback (most recent call last):
        ...
        TypeError: Poly_Gen_Relation unhashable
        """
        raise TypeError('Poly_Gen_Relation unhashable')

    def implies(self, Poly_Gen_Relation y):
        r"""
        Test whether ``self`` implies ``y``.

        INPUT:

        - ``y`` -- a :class:`Poly_Gen_Relation`.

        OUTPUT:

        Boolean. ``True`` if and only if ``self`` implies ``y``.

        Examples:

        >>> from ppl import Poly_Gen_Relation
        >>> nothing = Poly_Gen_Relation.nothing()
        >>> nothing.implies( nothing )
        True
        """
        return self.thisptr.implies(y.thisptr[0])

    @classmethod
    def nothing(cls):
        r"""
        Return the assertion that says nothing.

        OUTPUT:

        A :class:`Poly_Gen_Relation`.

        Examples:

        >>> from ppl import Poly_Gen_Relation
        >>> Poly_Gen_Relation.nothing()
        nothing
        """
        return _wrap_Poly_Gen_Relation(PPL_Poly_Gen_Relation_nothing())

    @classmethod
    def subsumes(cls):
        r"""
        Return the assertion "Adding the generator would not change
        the polyhedron".

        OUTPUT:

        A :class:`Poly_Gen_Relation`.

        Examples:

        >>> from ppl import Poly_Gen_Relation
        >>> Poly_Gen_Relation.subsumes()
        subsumes
        """
        return _wrap_Poly_Gen_Relation(PPL_Poly_Gen_Relation_subsumes())

    def ascii_dump(self):
        r"""
        Write an ASCII dump to stderr.

        Examples:

        >>> cmd  = 'from ppl import Poly_Gen_Relation\n'
        >>> cmd += 'Poly_Gen_Relation.nothing().ascii_dump()\n'
        >>> from subprocess import Popen, PIPE
        >>> import sys
        >>> proc = Popen([sys.executable, '-c', cmd], stderr=PIPE)
        >>> out, err = proc.communicate()
        >>> print(str(err.decode('ascii')))
        NOTHING
        >>> proc.returncode
        0
        """
        self.thisptr.ascii_dump()

    def __repr__(self):
        r"""
        Return a string representation.

        OUTPUT:

        String.

        Examples:

        >>> from ppl import Poly_Gen_Relation
        >>> Poly_Gen_Relation.nothing().__repr__()
        'nothing'
        """
        if self.implies(Poly_Gen_Relation.subsumes()):
            return 'subsumes'
        else:
            return 'nothing'

cdef _wrap_Poly_Gen_Relation(PPL_Poly_Gen_Relation relation):
    """
    Wrap a C++ ``PPL_Poly_Gen_Relation`` into a Cython ``Poly_Gen_Relation``.
    """
    cdef Poly_Gen_Relation rel = Poly_Gen_Relation(True)
    rel.thisptr = new PPL_Poly_Gen_Relation(relation)
    return rel



import unittest

from ppl import Variable, Constraint, Constraint_System

class TestConstraint(unittest.TestCase):
    def test_creation_empty(self):
        c = Constraint()
        self.assertTrue(c.is_tautological())

    def test_creation_other(self):
        x = Variable(0)
        y = Variable(1)
        c = x + 3 * y == 1
        cc = Constraint(c)
        self.assertTrue(c.is_equivalent_to(cc))

    def test_creation_invalid(self):
        self.assertRaises(TypeError, Constraint, "hello")

class TestConstraint_System(unittest.TestCase):
    def test_creation_empty(self):
        cs = Constraint_System()
        self.assertTrue(cs.empty())

    def test_creation_other(self):
        x = Variable(0)
        y = Variable(1)
        cs = Constraint_System(5*x - 2*y > 0)
        cs.insert(6 * x < 3 * y)
        ccs = Constraint_System(cs)
        self.assertTrue(cs[0].is_equivalent_to(ccs[0]))

    def test_creation_invalid(self):
        self.assertRaises(TypeError, Constraint_System, "hello")



#!/usr/bin/env python

import os
import sys

from setuptools import setup, Command
from setuptools.extension import Extension

# NOTE: setuptools build_ext does not work properly with Cython code
from distutils.command.build_ext import build_ext as _build_ext

# Adapted from Cython's new_build_ext
class build_ext(_build_ext):
    def run(self):
        # Check dependencies
        try:
            from Cython.Build.Dependencies import cythonize
        except ImportError as E:
            sys.stderr.write("Error: {0}\n".format(E))
            sys.stderr.write("The installation of ppl requires Cython\n")
            sys.exit(1)

        try:
            # We need the header files for cysignals at compile-time
            import cysignals
        except ImportError as E:
            sys.stderr.write("Error: {0}\n".format(E))
            sys.stderr.write("The installation of ppl requires cysignals\n")
            sys.exit(1)

        try:
            # We need the header files for gmpy2 at compile-time
            import gmpy2
        except ImportError as E:
            sys.stderr.write("Error: {0}\n".format(E))
            sys.stderr.write("The installation of ppl requires gmpy2\n")
            sys.exit(1)

        self.extensions[:] = cythonize(
            self.extensions,
            include_path=sys.path,
            compiler_directives={'embedsignature': True,
                                 'language_level': '3'})

        _build_ext.run(self)

class TestCommand(Command):
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        import subprocess, os, tempfile, shutil

        old_path = os.getcwd()
        tempdir_path = tempfile.mkdtemp()
        try:
            shutil.copytree('./tests', tempdir_path, dirs_exist_ok=True)
            os.chdir(tempdir_path)

            if subprocess.call([sys.executable, 'runtests.py']):
                raise SystemExit("Doctest failures")

            if subprocess.call([sys.executable, 'setup.py', 'build_ext', '--inplace']) or \
                    subprocess.call([sys.executable, '-c', "import testpplpy; testpplpy.test(); testpplpy.example()"]):
                raise SystemExit("Cython test 1 failure")

            if subprocess.call([sys.executable, 'setup2.py', 'build_ext', '--inplace']) or \
                    subprocess.call([sys.executable, '-c', "import testpplpy2; testpplpy2.test(); testpplpy2.example()"]):
                raise SystemExit("Cython test 2 failure")
        finally:
            os.chdir(old_path)
            shutil.rmtree(tempdir_path)

extensions = [
    Extension('ppl.linear_algebra', sources=['ppl/linear_algebra.pyx', 'ppl/ppl_shim.cc']),
    Extension('ppl.mip_problem', sources=['ppl/mip_problem.pyx', 'ppl/ppl_shim.cc']),
    Extension('ppl.polyhedron', sources = ['ppl/polyhedron.pyx', 'ppl/ppl_shim.cc']),
    Extension('ppl.generator', sources = ['ppl/generator.pyx', 'ppl/ppl_shim.cc']),
    Extension('ppl.constraint', sources = ['ppl/constraint.pyx', 'ppl/ppl_shim.cc']),
    Extension('ppl.congruence', sources=['ppl/congruence.pyx', 'ppl/ppl_shim.cc']),
    Extension('ppl.bit_arrays', sources = ['ppl/bit_arrays.pyx', 'ppl/ppl_shim.cc']),
    ]

setup(
    ext_modules = extensions,
    cmdclass = {'build_ext': build_ext, 'test': TestCommand},
)



# distutils: language = c++
# distutils: libraries = gmp ppl m
#*****************************************************************************
#       Copyright (C) 2018 Vincent Delecroix <vincent.delecroix@labri.fr>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#  as published by the Free Software Foundation; either version 3 of
#  the License, or (at youroption) any later version.
#                  http://www.gnu.org/licenses/
#*****************************************************************************

from libc.limits cimport ULONG_MAX

from .ppl_decl cimport PPL_Bit_Row

cdef class Bit_Row(object):
    r"""
    A bit row

    This can be considered as a subset of the non-negative integers (with
    upper bound the maximum value ``ULONG_MAX`` that fits in an
    ``unsigned long``).
    """
    def __cinit__(self):
        self.thisptr = new PPL_Bit_Row()

    def __dealloc__(self):
        del self.thisptr

    def __repr__(self):
        r"""
        Examples:

        >>> from ppl import Bit_Row
        >>> r = Bit_Row()
        >>> r
        {}
        >>> r.set(2); r.set(12)
        >>> r
        {2, 12}
        """
        cdef list l = []
        cdef unsigned long k = self.thisptr.first()
        while k != ULONG_MAX:
            l.append(str(k))
            k = self.thisptr.next(k)
        return "{" + ", ".join(l) + "}"

    def __hash__(self):
        r"""
        Examples:

        >>> from ppl import Bit_Row
        >>> r = Bit_Row()
        >>> hash(r)
        Traceback (most recent call last):
        ...
        TypeError: Bit_Row unhashable
        """
        raise TypeError("Bit_Row unhashable")

    def set(self, unsigned long k):
        r"""
        Set the bit in position ``k``

        Examples:

        >>> from ppl import Bit_Row
        >>> r = Bit_Row()
        >>> r.set(2); r.set(34); r.set(12)
        >>> r
        {2, 12, 34}
        >>> r.set(22)
        >>> r
        {2, 12, 22, 34}
        """
        self.thisptr.set(k)

    def set_until(self, unsigned long k):
        r"""
        Set bits up to position ``k`` (excluded)

        Examples:

        >>> from ppl import Bit_Row
        >>> r = Bit_Row()
        >>> r.set_until(5)
        >>> r
        {0, 1, 2, 3, 4}
        """
        self.thisptr.set_until(k)

    def clear_from(self, unsigned long k):
        r"""
        Clear bits from position ``k`` (included) onward

        Examples:

        >>> from ppl import Bit_Row
        >>> r = Bit_Row()
        >>> r.set_until(10)
        >>> r.clear_from(5)
        >>> r
        {0, 1, 2, 3, 4}
        """
        self.thisptr.clear_from(k)

    def clear(self):
        r"""
        Clear all the bits of the row

        Examples:

        >>> from ppl import Bit_Row
        >>> r = Bit_Row()
        >>> r.set(1); r.set(3)
        >>> r
        {1, 3}
        >>> r.clear()
        >>> r
        {}
        """
        self.thisptr.clear()

    def first(self):
        r"""
        Return the index of the first set bit or ``-1`` if no bit is set.

        Examples:

        >>> from ppl import Bit_Row
        >>> r = Bit_Row()
        >>> r.first() == -1
        True
        >>> r.set(127)
        >>> r.first() == 127
        True
        >>> r.set(2)
        >>> r.first() == 2
        True
        >>> r.set(253)
        >>> r.first() == 2
        True
        >>> r.clear()
        >>> r.first() == -1
        True
        """
        cdef unsigned long k = self.thisptr.first()
        if k == ULONG_MAX:
            return -1
        else:
            return k

    def last(self):
        r"""
        Return the index of the last set bit or ``-1`` if no bit is set.

        Examples:

        >>> from ppl import Bit_Row
        >>> r = Bit_Row()
        >>> r.last() == -1
        True
        >>> r.set(127)
        >>> r.last() == 127
        True
        >>> r.set(2)
        >>> r.last() == 127
        True
        >>> r.set(253)
        >>> r.last() == 253
        True
        >>> r.clear()
        >>> r.last() == -1
        True
        """
        cdef unsigned long k = self.thisptr.last()
        if k == ULONG_MAX:
            return -1
        else:
            return k

cdef class Bit_Matrix(object):
    pass



"""
The goal of this file is to test cython can use pplpy package properly
In order to do this we do some test with objects from each packages and extension :
ppl
ppl.linear_algebra
ppl.linear_algebra
ppl.constraint
ppl.mip_problem
"""

from ppl.linear_algebra cimport Variable
from ppl.constraint cimport Constraint_System
from ppl.mip_problem cimport MIP_Problem
from ppl.polyhedron cimport C_Polyhedron

def test():
    x = Variable(0)
    y = Variable(1)
    cs = Constraint_System()
    cs.insert( x >= 0)
    cs.insert( y >= 0 )
    cs.insert( 3 * x + 5 * y <= 10 )
    m = MIP_Problem(2, cs, x + y)
    m.objective_function()

    from ppl import C_Polyhedron
    C_Polyhedron( 5*x-2*y >=  x+y-1 )

    print("-"*80)
    print("Cython test 2 OK")
    print("-"*80)

def example():
    "Cython version of the example from the README"
    cdef Variable x = Variable(0)
    cdef Variable y = Variable(1)
    cdef Variable z = Variable(2)
    cdef Constraint_System cs = Constraint_System()
    cs.insert(x >= 0)
    cs.insert(y >= 0)
    cs.insert(x + y + z == 1)
    cdef C_Polyhedron poly = C_Polyhedron(cs)
    print(poly.minimized_generators())
    print('dim = %lu' % poly.thisptr.space_dimension())
    print("-"*80)
    print("Cython example 2 OK")
    print("-"*80)



import unittest

from ppl import Variable

class TestVariable(unittest.TestCase):
    def test_creation_valid(self):
        Variable(0)
    
    def test_creation_invalid(self):
        self.assertRaises(OverflowError, Variable, -1)
        self.assertRaises(TypeError, Variable, "hello")
